
lrfRun = function(
  Dat = NULL,                  #  A data frame containing the numeric columns:
                               #    x & y: location coordinates,
                               #    z: concentration values
  OutFile  = "lrf",            #  Output file base name
  Bdy,                         #  Set from which mass pts are drawn; the results of a
                               #    call to the function "BBox".
                               #    The default is to take the range of x and y and
                               #    expand by 10%.
  ItnMax  = 1e6,               #  Maximum iterations after burn-in
  ItnBrn  =  1e5,               #  Burn-in iterations
  JmpMax  = 500,               #  Maximum num. of LRF mass points to draw during the
                               #    creation of the initial levy process
                               #    (initialization of the Markov chain).
  TimMax  = 600,               #  Maximum computation time (not yet implemented)
  SavMax  = min(1e4,ItnMax),    #  Number of times to save state
  pX = c(pre=.10,bet=.00,add=.25,gam=.50,rem=.15),      #  Move Prob's
                               #    pre: precision
                               #    beta: not yet implemented: should be zero
                               #    add: add an atom
                               #    gam: move an atom
                               #    rem: remove an atom
  dX = c(dnew=1.0,dmvmag=0.5,dmvloc=1.0),  #  Step sizes for the move types.
                               #    dnew: scale of proposal for new atom
                               #    dmvmag: scale of step for a proposed change in atom
                               #      magnitude
                               #    dmvloc: scale of step for a proposed change in atom
                               #      location
  PriPar  = list(apre = 3,       #  Pre ~ Ga(apre,rpre) (the observation precision)
                 rpre = 1,
                 dist = 0.5,         #  Kernel distance param. in original units (e.g. degrees)
                 agam = 3,  
                 rgam = 1),        #  LRF   ~ Ga(agam dx, rgam)
                                           #    LRF is the location and
                                           #    weight of one of the atoms (gamma_m)
  frac = 0.010,                # Fraction of Gamma mass to omit (=> eps)
  printStats = T               # Whether to print summary statistics about the MCMC.
  )
{
#######################################################################
#
# Runs the MCMC, given a data set and bounding region.
#

  
  #   Check the inputs:
  if(any(pX<0)||sum(pX)==0) stop("lrfRun: Probs pX must be >=0");
  else  names(pX) = c("pre", "bet", "add", "gam", "rem");
  if(any(dX<0)) stop("lrfRun: Step size dX must be >=0");
  else  names(dX) = c("dnew", "dmvmag", "dmvloc");

  if( missing(Bdy) || !inherits(Bdy,"Poly") ) {
    if( is.null( Dat ) ){
      xlim = c(0,1);
      ylim = c(0,1);
      Bdy = BBox( x = xlim, y = ylim );
    } else {
      Bdy = BBox(Dat$x, Dat$y, expa=0.10, Geo= F);
      xlim = Bdy@xlim;
      ylim = Bdy@ylim;
    }
  } else {
    xlim = Bdy@xlim;
    ylim = Bdy@ylim;
  }
  
  #   Save the function arguments and environment info
  MyId      = getId();         #  General info about user & env.
  MyId$Call = deparse(match.call());  #  Get arguments from invocation
  MyId$Args = args(lrfRun)     #  Default values of other args
  MyId$Data = Dat;          #  Remember the input file
  MyId$dX   = dX;              #  Remember the step sizes

  #   Save information about the Gamma(dx) prior and the number of data pts
  MyId$etc=list(
       frac = frac,
       N    = length(Dat$x),             # How many data points?
       bdy  = Bdy                           # Draw LRF from what polygon? 
    )
  MyId$Pri = PriPar;                        # Prior parameters
  
  #   Handle time-keeping
  TimMax = dhms2s(casefold(TimMax));          # Expand "mins" etc. to secs

  #   Print this information to file
  OutFileLG = paste(OutFile,"Out.LG",sep="");   # Output Log (dput/dget)
  OutFileFL = paste(OutFile,"Out.FL",sep="");   # Output, Fixed length.
                                              # This will contain the meta-data
                                              # for the output.  
  OutFileVL = paste(OutFile,"Out.VL",sep="");   # Output, Variable length
                                              # This will contain the samples.
  unlink(c("OutFileFL","OutFileVL","OutFileLG"));
  dput(MyId,OutFileLG);

  #   Run the MCMC, saving samples to files
  lrfMCMC.c( Dat = Dat, OutFileFL = OutFileFL,
          OutFileVL = OutFileVL, xlim = xlim, ylim = ylim,
          ItnMax = ItnMax, ItnBrn = ItnBrn,
          JmpMax = JmpMax, TimMax = TimMax, SavMax = SavMax, 
          pX = pX, dX = dX, PriPar = PriPar,
          frac = frac, printStats = printStats );

  #   Return the call info
  invisible(MyId);
}


  

lrfMCMC.c = function( 
  Dat,                      #  A data frame containing the numeric columns:
                               #    x & y: location coordinates,
                               #    z: concentration values
  OutFileFL,                    #  The filename of the fixed length file to be created
  OutFileVL,                    #  The filename of the variable length file to be created
  xlim, ylim,                  #  The boundaries of the set from which mass pts are drawn
  ItnMax,                      #  Maximum iterations
  ItnBrn,                      #  Burn-in iterations
  JmpMax,                      #  Maximum num. of LRF mass points
  TimMax,                      #  Maximum time, in seconds
  SavMax,                      #  Number of times to save state
  pX,                          #  Move Prob's
  dX,                          #  Step sizes
  PriPar,                      #  Hyperparameters for prior specification
  frac,                        #  Fraction of Gamma mass to omit
  printStats                   #  Whether to print summary statistics about the MCMC
  )
{
#######################################################################
#
# Internal use only.  Calls a C routine to runs the MCMC, given a data
#   set and bounding region.
#

  if( is.null( Dat ) ){
    .C("MCMC",
       datX = as.double(c()),                datY = as.double(c()),
       datZ = as.double(c()),                datLen = as.integer( 0 ),
       OutFL = OutFileFL,                     OutVL = OutFileVL,                    
       xlim = as.double(xlim),               ylim = as.double(ylim),
       ItnMax = as.integer(ItnMax),          ItnBrn = as.integer(ItnBrn),
       JmpMax = as.integer(JmpMax),          TimMax = as.integer(TimMax),
       SavMax = as.integer(SavMax),          pX = as.double(pX),
       dX = as.double(dX),                   apre = as.double(PriPar@apre),
       rpre = as.double(PriPar@rpre),        dist = as.double(PriPar@dist),
       agam = as.double(PriPar@agam),        rgam = as.double(PriPar@rgam),
       frac = as.double(frac),               printStats = as.integer(printStats),
       COPY = rep(FALSE,22) );
  } else {
    .C("MCMC",
       datX = as.double(Dat$x),           datY = as.double(Dat$y),
       datZ = as.double(Dat$z),           datLen = as.integer( length(Dat$y) ),
       OutFL = OutFileFL,                     OutVL = OutFileVL,                    
       xlim = as.double(xlim),               ylim = as.double(ylim),
       ItnMax = as.integer(ItnMax),          ItnBrn = as.integer(ItnBrn),
       JmpMax = as.integer(JmpMax),          TimMax = as.integer(TimMax),
       SavMax = as.integer(SavMax),          pX = as.double(pX),
       dX = as.double(dX),                   apre = as.double(PriPar@apre),
       rpre = as.double(PriPar@rpre),        dist = as.double(PriPar@dist),
       agam = as.double(PriPar@agam),        rgam = as.double(PriPar@rgam),
       frac = as.double(frac),               printStats = as.integer(printStats),
       COPY = rep(FALSE,22));
  }
}

