
#######################################################################
  setClass("Vec", representation(x="numeric",y="numeric",len="integer"));
  Vec = function(x,y)  {
  # a vector... 2dim in our applications
     if(missing(y)) {
       if(is.matrix(x)&&ncol(x)>1) {
         y = x[,2];
         x = x[,1];
       } else
       stop("Vec: x must be matrix with >=2 cols if y missing");
     }
     x   = as.vector(x);
     y   = as.vector(y);
     len = length(x);
     if(length(y)!=len) stop("Vec: x,y lengths must be equal");
     new("Vec", x=x, y=y, len=len);
  }
  validVec = function(object) {
    ok = !any(is.na(match(c("x","y","len"),slotNames(object))));
    ok && identical(length(object),length(object@x)) &&
          identical(length(object),length(object@y));
  }
  setValidity("Vec",validVec);
  setMethod("length", signature(x="Vec"),
    function(x) { x@len }
  )
  closed = function(xy) {
       diff(xy@x[c(1,length(xy))])==0 &&
       diff(xy@y[c(1,length(xy))])==0;
  }
  lrfAppend = function(x,values) {
       x@x = c(x@x, values@x);
       x@y = c(x@y, values@y);
       x@len = x@len + values@len;
       x;
  }
#######################################################################
  setClass("Meas", representation("Vec", z="numeric"));
  Meas = function(xy, z, dm) {
    if(!extends(class(xy),"Vec")) stop("Meas: Expecting Vec");
    if(missing(z)) z = rep(1,length(xy));  # Default: Unit weights
    if(length(z)!=length(xy)) stop("Meas: lengths must be equal");
    if(!missing(dm) && validMeas(dm)) {    # Extend earlier measure
      xy = Vec(c(dm@x,xy@x),c(dm@y,xy@y));  z = c(dm@z,z);
    }
    new("Meas", xy, z=z);
  }
  setMethod("Meas", signature(xy="matrix"),
            function(xy, z, dm) Meas(Vec(xy), z) )
  validMeas = function(object=NULL) {
    ok = !any(is.na(match(c("x","y","z","len"),slotNames(object))));
    ok && identical(length(object),length(object@x)) &&
          identical(length(object),length(object@y)) &&
          identical(length(object),length(object@z));
  }
  setMethod("show", "Meas",
    function(object) {
      symbols(object@x, object@y,circles=sqrt(object@z));
      show(unclass(object));
    }
  )
#######################################################################
  setClass("Poly", representation("Vec", mu="Vec", prob="numeric",
                   xlim="numeric", ylim="numeric", area="numeric",
                                   Geo="logical",));
# removeMethods("Poly");
  Poly = function(xy, Geo) {                 # Convert Vec to Poly
    if(missing(Geo))                         # USA lat & lon:
      Geo = (-124.7 < min(xy@x) & max(xy@x) < -67.0 &
                  25.1 < min(xy@y) & max(xy@y) <  49.4);
    # Close up perimeter of xy list, if needed:
    if(!closed(xy))  xy = lrfAppend(xy,Vec(xy@x[1],xy@y[1]));
    # Compute bounding box:
    xlim = range(xy@x, na.rm=T);
    ylim = range(xy@y, na.rm=T);
    # Area and area-weighted center of polygon:
    N    = xy@len;
    midx = (xy@x[-1]+xy@x[-N])/2;  dx = (xy@x[-1]-xy@x[-N]);
    midy = (xy@y[-1]+xy@y[-N])/2;  dy = (xy@y[-1]-xy@y[-N]);
    #   wgt is signed area of triangle from 0 to segments
    #   wgt  = xy@x[-N]*xy@y[-1] - xy@x[-1]*xy@y[-N], better as:
    wgt  = xy@x[-N]*dy - dx*xy@y[-N];
    area = sum(wgt)/2;
    mu   = Vec(wgt%*%midx/(3*area), wgt%*%midy/(3*area));
    #   tmp  is signed area of triangle from mu to points:
    tmp  = mu@x * xy@y - xy@x * mu@y;
    #   prob is (positive!) area of triangle from mu to segments:
    prob = (wgt + tmp[-N] - tmp[-1])/2;
    if(Geo) area = GeoArea(xy@x, xy@y);
    new("Poly", xy, mu=mu, prob=prob, xlim=xlim, ylim=ylim, area=area,
        Geo=Geo);
  }
  inPoly = function(poly, xy)  {  # Not (yet?) vector in x,y 
    if(extends(class(poly),"Vec")) poly = Poly(poly);
    if (!extends(class(poly),"Poly")) stop("inPoly: First arg must be Poly");
    n  = length(poly);
    ox = poly@x[-n]; nx = poly@x[-1];
    oy = poly@y[-n]; ny = poly@y[-1];
    cross = ((sign(ny-xy@y) != sign(oy-xy@y))) &  #  Cross horizontal line?
    (sign((ox-xy@x)*(ny-xy@y)-(nx-xy@x)*(oy-xy@y))!=sign(ny-oy));  # On left?
    sum(cross) %% 2;                        #  Odd # of crossings -> Inside
  }
  CCW = function(a,b,c)        {  # Are three points in counterclockwise order?
    (c@y-a@y)*(b@x-a@x) > (b@y-a@y)*(c@x-a@x);
  }
  cross = function(a,b,c,d){  # Do segments (a,b) and (c,d) cross?
    (CCW(a,b,d)!=CCW(b,c,d)) && (CCW(a,b,c) != CCW(a,b,d));
  }
  refl = function(poly, oxy, nxy)  {
    if(extends(class(poly),"Vec")) poly = Poly(poly);
    if(!extends(class(poly),"Poly")) stop("refl: First arg must be Poly");
    if(!inPoly(poly, oxy)) stop("refl: oxy must be inside Poly");
    n  = length(poly);
    while(!inPoly(poly, nxy)) {    # Reflect repeatedly at boundary
      ax = poly@x[n]; ay = poly@y[n]; axy = Vec(ax,ay);
      bx = poly@x[1]; by = poly@y[1]; bxy = Vec(bx,by);
      for(i in 2:n) {
        if(cross(oxy,nxy, axy,bxy)) {
          # Replace nxy with reflection; replace oxy with intersection.
        }
        # replace axy with bxy; replace bxy with next point, xy[i]
      }
    }
    # return nxy as reflection inside polygon.
  }
  rPoly = function(N=1, poly) {
  ##############################################################
  #
  # Internal use only.  Not currently utilized.
  #
  # Draw N random points uniformly from interior of polygon
    
    if(missing(poly)) poly = Poly(Vec(x=c(0,1,1,0,0),y=c(0,0,1,1,0)));
    if(extends(class(poly),"Vec")) poly = Poly(poly);
    # Pick N random triangles, with probs proportional to area:
    tri = sample(poly@len-1, N, replace=T, prob=poly@prob);
    # Need uniforms U,V for each triangle, with U < V:
    # Yields (2 x len) matrix (think of it as "rbind(U,V)")
    uv = apply(matrix(runif(2*N),N,2),1,range);
    x  = poly@x[tri]   * (1-uv[2,]) +
         poly@x[tri+1] * apply(uv,2,diff) + poly@mu@x * uv[1,];
    y  = poly@y[tri]   * (1-uv[2,]) +
         poly@y[tri+1] * apply(uv,2,diff) + poly@mu@y * uv[1,];
    Vec(x,y);
  }
  setMethod("plot", signature(x="Poly"),
     function(x, y, ...) {
       plot(x@x,x@y,type="l",xlab="x",ylab="y", ...);
       points(x@mu@x,x@mu@y,pch=3);
    }
  )
  validPoly = function(object=NULL) {
    ok = !any(is.na(match(c("x","y","len","mu","prob","xlim",
                            "ylim","area"),slotNames(object))));
    ok && identical(length(object),length(object@x)) &&
          identical(length(object),length(object@y)) &&
          validVec(object@mu) && 
          identical(length(object)-1,length(object@prob)) &&
          all(0<=object@prob);
  }
  BBox = function(x, y, expand=0, Geo=FALSE) {
  ##############################################################
  #
  # Create a polynomial representing the bounding box containing
  # the data points.  
  #

    xl = range(x); xl = xl + expand * diff(xl) * c(-1,1);
    yl = range(y); yl = yl + expand * diff(yl) * c(-1,1);
    Poly(Vec(xl[c(1,2,2,1,1)], yl[c(1,1,2,2,1)]), Geo);
  }
#######################################################################
  setClass("Lrf", representation("Meas", bdy="Poly", eps="numeric",
                                 tau="numeric"));
  Lrf = function(dm, bdy, eps, tau) {
    new("Lrf", dm, bdy=bdy, eps=eps, tau=tau);
  }
  setMethod("Lrf", signature(dm="Meas"),
            function(dm, bdy, eps, tau) {
              if(missing(bdy)) bdy = Poly(Vec(cxhull(dm@x,dm@y,0.05)));
              if(missing(eps)) eps = min(dm@z);
              if(missing(tau)) tau = 0.0;
              new("Lrf", dm, bdy=bdy, eps=eps, tau=tau);
            }
  )
  setMethod("Lrf", signature(dm="Vec"),       # Note bdy eps & tau
            function(dm, bdy, eps, tau) {     # MUST BE missing
              Lrf(Meas(dm));
            }
  )
  LrfAdd = function(lrf,xy,z) {               # Add 1 or more elt's
    if(!extends(class(lrf),"Lrf"))  stop("LrfAdd: Expecting Lrf");
    if(!extends(class(xy),"Vec"))   stop("LrfAdd: Expecting Vec");
    if(!inPoly(lrf@bdy,xy))         warning("LrfAdd: xy outside bdy");
    if(any(z < lrf@eps))            warning("LrfAdd: z below eps");
    Lrf(Meas(xy,z,lrf),lrf@bdy,lrf@eps,lrf@tau);
  }
  LrfRem = function(lrf,j) {                  # Remove 1 or more elt's
    if(!extends(class(lrf),"Lrf"))  stop("LrfRem: Expecting Lrf");
    if(any(!is.integer(j) | j<1 | j>length(lrf)))
      stop(paste("LrfRem: Bad j =",j,"(M =",length(lrf)));
    lrf@x = lrf@x[-j]; lrf@y = lrf@y[-j]; lrf@z = lrf@z[-j];
    lrf@len = length(lrf) - length(j);
    lrf;
  }
  LrfMov = function(lrf,j,dxy,z) {            # Move 1 or more elt's
    R   = 6378.137;      # Earth's radius in km.
    if(!extends(class(lrf),"Lrf"))  stop("LrfMov: Expecting Lrf lrf");
    if(!extends(class(dxy),"Vec"))  stop("LrfMov: Expecting Vec dxy");
    if(any(!is.integer(j) | j<1 | j>length(lrf)))
      stop(paste("LrfRem: Bad j =",j,"(M =",length(lrf)));
    if(length(j)!=length(dxy) | length(j)!=length(z))
                                    stop("LrfMov: Length mismatch");
    if(lrf@bdy@Geo) {    # Approximate diffs in degrees for dxy in km
      x = lrf@x[j] + dxy@x * 180/(pi*R) /cos(lrf@y[j]*pi/180);
      y = lrf@y[j] + dxy@y * 180/(pi*R);
    } else {
      x = lrf@x[j] + dxy@x;  y = lrf@y[j] + dxy@y;
    }
    # Want to reflect xy into polygon; haven't worked out details
    # for general convex polygon, so instead we use BBox for now:
    xlim = lrf@bdy@xlim;
    while(x<xlim[1] || x>xlim[2]) {
      if(x<xlim[1]) x = 2*xlim[1] - x;   if(x>xlim[2]) x = 2*xlim[2] - x;
    }
    ylim = lrf@bdy@ylim;
    while(y<ylim[1] || y>ylim[2]) {
      if(y<ylim[1]) y = 2*ylim[1] - y;   if(y>ylim[2]) y = 2*ylim[2] - y;
    }    
    if(!inPoly(lrf@bdy,Vec(x,y)))   warning("LrfMov: New xy outside bdy");
    if(any(z < lrf@eps))            warning("LrfMov: z below eps");
    lrf@x[j] = x; lrf@y[j] = y; lrf@z[j] = z;
    lrf;
  }
  validLrf = function(object) {
    ok = !any(is.na(match(c("x","y","z","len","bdy","eps"),
                          slotNames(object))));
    ok && identical(length(object),length(object@x)) &&
          identical(length(object),length(object@y)) &&
          identical(length(object),length(object@z)) &&
          (object@tau >= 0);
  }
  setValidity("Lrf",validLrf);
  setMethod("show", "Lrf",
     function(object) {
       if(object@bdy@Geo) {
         xlab="x";         ylab="y";
         scale = 1.0;
       } else if(abs(mean(object@x,na.rm=T))>1000) {
         xlab="Easting (1000's)";  ylab="Northing (1000's)";
         scale = 1000;
       } else {
         xlab="X";         ylab="Y";
         scale = 1.0;
       }
       x = object@x/scale; y = object@y/scale; z = object@z/scale;
       bdy  = list(x=object@bdy@x/scale,y=object@bdy@y/scale,
              mu=Vec(object@bdy@mu@x/scale,object@bdy@mu@y/scale));
       big  = 0.50;   # Biggest circle, in inches
       opar = par();
       xl   = range(x,bdy@x);
       yl   = range(y,bdy@y);
       dx   = big*diff(xl)/opar$pin[1];
       dy   = big*diff(yl)/opar$pin[2];
       xl   = c(xl[1]-dx/2,xl[2]+dx/2);
       yl   = c(yl[1]-dy/2,yl[2]+dy/2);
       plot(bdy@x, bdy@y, type="n", xlim=xl, ylim=yl,
            xlab=xlab,ylab=ylab,cex=1.75);
       symbols(x,y, circles=sqrt(z), inches=big, add=T);
       points(x,y);
       points(bdy@mu@x, bdy@mu@y, pch=3);
       lines(bdy@x ,bdy@y, lty=3, col=2);
#       print(summary(unclass(object)));
    }
  )
#######################################################################
cxhull = function(x,y, expand=0) {
    # Generates ccw polyline traversing convex hull of points [x,y];
    # if < 3 points, gives bounding box.
     if(missing(y)) {
       if(is.matrix(x)&&ncol(x)>1) {
         ok = !is.na(x[,1]) & !is.na(x[,2]);
         y = x[ok,2];
         x = x[ok,1];
       } else {
       print(paste("cxhull: class of x is:",class(x)));
       stop('Vec: x must be matrix with >=2 cols if y missing');
           }
     }
     if (!identical(length(x), length(y)))
       stop("Vec: x,y must have same length");
     if (length(x)<3) {
       xr = range(x, na.rm=T);  yr = range(y, na.rm=T);
       bx = xr[c(1,2,2,1,1)];
       by = yr[c(1,1,2,2,1)];
       val = cbind(bx,by);
     } else {
       cxh = rev(chull(x,y));    # Switch from cw to ccw
       cxh = c(cxh,cxh[1]);      # Close up the loop
       bx  = x[cxh];
       by  = y[cxh];
       val = cbind(bx,by);
     }
     if(expand>0) {
       mu  = apply(val,2,mean);
       val = mu + (val-mu)*(1+expand);
     }
     val;
   }
#######################################################################
  setClass("Dat", representation(x="numeric",y="numeric",
                                 z="numeric",len="integer"));
  Dat = function(x,y,z)  {
     if(missing(z)) {
       if(is.matrix(x)&&ncol(x)>=3) {
         z = x[,3]; y = x[,2]; x = x[,1];
       } else
       stop("Dat: x must be matrix with >=3 cols if y,z missing");
     }
     x = as.vector(x);
     y = as.vector(y);
     z = as.vector(z);
     len = length(x);
     if(length(y)!=len || length(z) != len)
       stop("Dat: y,x,z lengths must be equal");
     new("Dat", x=x, y=y, z=z, len=len);
  }
  validDat = function(object) {
    ok = !any(is.na(match(c("x","y","z","len"),slotNames(object))));
    ok && identical(length(object),length(object@x)) &&
          identical(length(object),length(object@y)) &&
          identical(length(object),length(object@z));
  }
  setValidity("Dat",validDat);
  setMethod("length", signature(x="Dat"),
    function(x) { x@len }
  )
  setMethod("plot", signature(x="Dat"),
     function(x, y, ...) {
       plot(x@x,x@y,type="n",xlab="x",ylab="y",...);
       symbols(x@x,x@y,circles=sqrt(x@z),inches=0.1,add=T);
    }
  )
  plot.Dat = function(x,...) {
    symbols(x@x,x@y,circles=sqrt(x@z),col=4,inches=0.1,...);
    map(database="state",region=c("del","mar","new j","new y","north c",
                  "pen","vir","wes"), xlim=range(x@x), ylim=range(x@y),
                  add=TRUE);
    #plot(x@x,x@y,type="n",xlab="x",ylab="y");
    }
#######################################################################

GeoArea = function(lon, lat) {
  # Compute area of closed spherical polygon given in degrees lon,lat.
  # Uses Girard's Theorem for polygon with n sides,
  #      Area = R^2 * { Sum of interior angles - (n-2) pi }
  # Tricky part is getting the angles... we use cross products.

  R   = 6378.137;      # Earth's radius in km.
  N   = length(lon);
  z   = sin(lat*pi/180);
  y   = cos(lat*pi/180) * sin(lon*pi/180);
  x   = cos(lat*pi/180) * cos(lon*pi/180);

  # Cross product of increments:
  xpx = y[-N]*z[-1] - y[-1]*z[-N];
  xpy = x[-1]*z[-N] - x[-N]*z[-1];
  xpz = x[-N]*y[-1] - x[-1]*y[-N];

  # Normalize to unit vectors, replicate 1st el't to last:
  len = sqrt(xpx^2 + xpy^2 + xpz^2);
  ux  = xpx / len;  ux = c(ux,ux[1]);
  uy  = xpy / len;  uy = c(uy,uy[1]);
  uz  = xpz / len;  uz = c(uz,uz[1]);

  # Cosines of successive angles:
  cth = ux[-N]*ux[-1] +  uy[-N]*uy[-1] +  uz[-N]*uz[-1];
  th  = acos(-cth);
  area= R^2 * (sum(th) - (N-3)*pi);
}
#######################################################################

GeoDis = function(lon1,lat1, lon2,lat2) {
  # Compute great-circle distance on earth; formula is equiv to:
  # d=R*acos(sin(lat1)*sin(lat2)+cos(lat1)*cos(lat2)*cos(lon1-lon2))  
  R   =  6378.137;      # Earth's radius in km.
  lon = c(lon1,lon2)*pi/180.0;  lat = c(lat1,lat2)*pi/180.0;
#  d   = 2*R*asin(sqrt((sin((lat[1]-lat[2])/2))^2 + 
#        cos(lat[1])*cos(lat[2])*(sin((lon[1]-lon[2])/2))^2));
  d   = 2*R*asin(sqrt((sin(diff(lat)/2))^2 + prod(cos(lat))*(sin(diff(lon)/2))^2));
}
