meanSurface <- function(
  McmcFile = "lrf",  # The base name for the files containing the
                     # posterior samples.
  x, y,              # The x and y coordinates of the points where
                     # the mean surface should be evaluated.
  SampNum = -1       # 
){
  #######################################################################
  #
  # For internal use only. 
  #

  DatLG = paste(McmcFile,"Out.LG",sep="");     # Log file
  if(!file.exists(DatLG))
    stop(paste("Unable to read lrf log file",DatLG));
  LogInfo  = dget(DatLG);                       # Misc stuff from log
  bdy  = LogInfo$etc$bdy;
  dist = LogInfo$Pri$dist;

  DatFL = paste(McmcFile,"Out.FL",sep="");     # Fixed-length data
  DatVL = paste(McmcFile,"Out.VL",sep="");     # Variable-length data
  if(!file.exists(DatFL))
    stop(paste("Unable to read fixed-length file",DatFL));
  if(!file.exists(DatVL))
    stop(paste("Unable to read vble-length file",DatVL));

  z = rep( 0, length( x ) );
  o <- .C("meanSurf",
          DatFL = DatFL,                  DatVL = DatVL,
          dist = as.double(dist),
          xlocs = as.double(x),       ylocs = as.double(y),
          z = as.double(z), nLocs = as.integer( length(x) ),
          sampNum = as.integer( SampNum - 1 ) );  # C indexes from zero
  
  return(o$z);
}

stdevSurface <- function(
  McmcFile = "lrf",  # The base name for the files containing the
                     # posterior samples.
  x, y               # The x and y coordinates of the points where
                     # the mean surface should be evaluated.
){
  #######################################################################
  #
  # For internal use only. 
  #

  DatLG = paste(McmcFile,"Out.LG",sep="");     # Log file
  if(!file.exists(DatLG))
    stop(paste("Unable to read lrf log file",DatLG));
  LogInfo  = dget(DatLG);                       # Misc stuff from log
  bdy  = LogInfo$etcs$bdy;
  dist = LogInfo$Pri$dist;

  DatFL = paste(McmcFile,"Out.FL",sep="");     # Fixed-length data
  DatVL = paste(McmcFile,"Out.VL",sep="");     # Variable-length data
  if(!file.exists(DatFL))
    stop(paste("Unable to read fixed-length file",DatFL));
  if(!file.exists(DatVL))
    stop(paste("Unable to read vble-length file",DatVL));

  z = rep( 0, length( x ) );
  o <- .C("stdevSurf",
          DatFL = DatFL,                  DatVL = DatVL,
          dist = as.double(dist),
          xlocs = as.double(x),       ylocs = as.double(y),
          z = as.double(z), nLocs = as.integer( length(x) ) );
  
  return(o$z);
}

percentAbove <- function(
  McmcFile = "lrf",  # The base name for the files containing the
                     # posterior samples.
  x, y,              # The x and y coordinates of the points where
                     # the surface should be evaluated.
  cutoff             # The cutoff for which the percent of samples
                     # exceeding should be calculated
){
  #######################################################################
  #
  # For internal use only. 
  #

  DatLG = paste(McmcFile,"Out.LG",sep="");     # Log file
  if(!file.exists(DatLG))
    stop(paste("Unable to read lrf log file",DatLG));
  LogInfo  = dget(DatLG);                       # Misc stuff from log
  bdy  = LogInfo$etc$bdy;
  dist = LogInfo$Pri$dist;

  DatFL = paste(McmcFile,"Out.FL",sep="");     # Fixed-length data
  DatVL = paste(McmcFile,"Out.VL",sep="");     # Variable-length data
  if(!file.exists(DatFL))
    stop(paste("Unable to read fixed-length file",DatFL));
  if(!file.exists(DatVL))
    stop(paste("Unable to read vble-length file",DatVL));

  z = rep( 0, length( x ) );
  o <- .C("percentAbove",
          DatFL = DatFL,                  DatVL = DatVL,
          dist = as.double(dist),
          xlocs = as.double(x),       ylocs = as.double(y),
          z = as.double(z), nLocs = as.integer( length(x) ),
          cutoff = as.double(cutoff) );
  
  return(o$z);
}

meanContour <- function(
  McmcFile = "lrf",  # The base name for the files containing the
                   # posterior samples.
  geo = FALSE,     # Whether to treat the data as geographical when plotting.
  nxy=100,         # The number of grid points for the prediction maps.
                   # The square root is the size of the grid on one side.
  ps="",           # The name of the postscript file to generate.  If ps="",
                   # then a graphical window is opened and no file is generated.
  county=NULL,
  ShowDat = F,     # Whether to add the data to the map
  MyDat = NULL     # If ShowDat = T, the data to be plotted on the contour plot.
){
  
  #######################################################################
  #
  # Create a contour plot of the posterior mean surface.
  # McmcFile is the base (root) name for the MCMC output files.
  # The MCMC files should be named <McmcFile>Out.FL and <McmcFile>Out.VL.  There
  # should also be a log file <McmcFile>Out.LG containing the log
  # information from the MCMC run.
  #

  win  = regMatch(casefold(getId()$os),"windows");  # Running under windows?
  DatLG = paste(McmcFile,"Out.LG",sep="");     # Log file
  if(!file.exists(DatLG))
    stop(paste("Unable to read lrf log file",DatLG));
  MyId  = dget(DatLG);                       # Misc stuff from log
  bdy  = MyId$etc$bdy;
  xl = bdy@xlim;
  yl = bdy@ylim;

  lam = setGrid(bdy, nreq=nxy);
  nx = length(lam$x);
  ny = length(lam$y);
  xx = rep(lam$x,rep(ny,nx));
  yy = rep(lam$y,nx);
  
  lam$z <- meanSurface(McmcFile = McmcFile, x = xx, y = yy);
  lam$z = matrix(lam$z, nrow=ny, ncol=nx);
  
  if(nchar(ps)) {
    postscript(paste(ps,"-c.eps",sep=""),hori=TRUE, color.p=TRUE);
    setAspect(bdy,0.95); }
  else if(!win)  { setAspect(bdy); }
  par(mar=c(5,5,4,2));
  if(geo) {
    xlab="Longitude";         ylab="Latitude";
  } else if(mean(abs(xl)) >1000) {
    xlab="Easting (1000's)";  ylab="Northing (1000's)";
  } else {
    xlab="X";         ylab="Y";
  }

  oldWarn <- getOption("warn");
  options(warn=-1);
  contour(lam,xlab=xlab,ylab=ylab,cex=2);

  if(geo) doMaps(c(xl,MyDat$x),c(yl,MyDat$y), area,
                       c("mary","penn","virg", "west"), county=county);
  if(ShowDat)  ShowDat(MyDat);
  options(warn=oldWarn);

  if(nchar(ps))
    dev.off();

  invisible(lam);
}


meanImage <- function(
  McmcFile = "lrf",  # The base name for the files containing the
                   # posterior samples.
  geo = FALSE,     # Whether to treat the data as geographical when plotting.
  nxy=100,         # The number of grid points for the prediction maps.
                   # The square root is the size of the grid on one side.
  ps="",           # The name of the postscript file to generate.  If ps="",
                   # then a graphical window is opened and no file is generated.
  county=F,        # Whether county boundaries should be shown
  ShowDat = F,     # Whether to add the data to the map
  MyDat = NULL,    # If ShowDat = T, the data to be plotted on the contour plot.
  zlim = NULL,     # For an image plot this can be used to specify the endpoints of
                   # the color range.
  col = NULL,      # The colors to be used in an image plot.  default is image.palette()
  SampNum = -1,    # Which sample in the MCMC should be plotted.  If SampNum == -1, then
                   # the mean over the samples is plotted.
  xlim = NULL,     # The xlim and ylim arguments can be used to override the scaling / 
  ylim = NULL,     # centering of the bounding box used by the MCMC for the graphical
                   # display.  For instance, the MCMC may have used units of tenths of 
                   # degrees, and the graphical display should perhaps be in units of
                   # degrees.  In such a case, set xlim and ylim to be the bounding box
                   # used by the MCMC in the desired units.  The translation is then
                   # done automatically.
  xlab = NULL,     # The x axis label.  If not specified, defaults are created based on
                   # the value of "geo" and the limits of the bounding box for the study area.
  ylab = NULL,     # The y axis label.  If not specified, defaults are created.
  main = NULL,     # The image title.  If not specified, a default is created.
  ...              # Additional graphical arguments for the call to image()
){
  
  #######################################################################
  #
  # Create an image plot of the posterior mean surface.
  # McmcFile is the base (root) name for the MCMC output files.
  # The MCMC files should be named <McmcFile>Out.FL and <McmcFile>Out.VL.  There
  # should also be a log file <McmcFile>Out.LG containing the log
  # information from the MCMC run.
  #

  win  = regMatch(casefold(getId()$os),"windows");  # Running under windows?
  DatLG = paste(McmcFile,"Out.LG",sep="");     # Log file
  if(!file.exists(DatLG))
    stop(paste("Unable to read lrf log file",DatLG));
  MyId  = dget(DatLG);                       # Misc stuff from log
  bdy  = MyId$etc$bdy;
  xl = bdy@xlim;
  yl = bdy@ylim;
  lam = setGrid(bdy, nreq=nxy);
  nx = length(lam$x);
  ny = length(lam$y);
  xx = rep(lam$x,rep(ny,nx));
  yy = rep(lam$y,nx);
  
  area = (xl[2] - xl[1]) * (yl[2] - yl[1]);
  lam$z <- meanSurface(McmcFile = McmcFile, x = xx, y = yy, SampNum = SampNum);
  lam$z = t(matrix(lam$z, nrow=ny, ncol=nx));
  # if lam$z is outside of the interval given by zlim, truncate.
  if( !missing( zlim ) ){
    lam$z = apply( lam$z, c(1,2), max, zlim[1] );
    lam$z = apply( lam$z, c(1,2), min, zlim[2] );
  }

  if(nchar(ps)) {
    postscript(paste(ps,"-i.eps",sep=""),hori=TRUE, color.p=TRUE);
    setAspect(bdy,0.95); }
  else if(!win)  { setAspect(bdy); }
  par(mar=c(5,5,4,2));

  if( !is.null( xlim ) && !is.null( ylim ) ){
    lam$x <- rescale( lam$x, xl, xlim );
    lam$y <- rescale( lam$y, yl, ylim );

    if( !is.null( MyDat ) ){
      MyDat$x <- rescale( MyDat$x, xl, xlim );
      MyDat$y <- rescale( MyDat$y, yl, ylim );
    }
  }

  if( is.null( xlab ) || is.null( ylab ) ){
    if(geo) {
      xlab="Longitude";         ylab="Latitude";
    } else if(mean(abs(xl)) >1000) {
      xlab="Easting (1000's)";  ylab="Northing (1000's)";
    } else {
      xlab="X";         ylab="Y";
    }
  }

  # Create the arguments for the image function call.
  # xaxs="i" and yaxs="i" specify that no extra whitespace
  # should be left around the image plot
  image.args = list( x=lam$x, y=lam$y, z = lam$z,
    xlab=xlab, ylab=ylab, cex=2,
    xaxs = "i", yaxs = "i",  
    ... );  
  if( !missing( zlim ) )
    image.args$zlim = zlim;
  if( !missing( col ) )
    image.args$col = col;
  image.args$xlim <- xlim;
  image.args$ylim <- ylim;
  do.call( "image", image.args);
  
  # the image plotting writes over the axis lines, so
  # redraw these lines
  abline(v=xlim[1], err=-1);
  abline(v=xlim[2], err=-1);
  abline(h=ylim[1], err=-1);
  abline(h=ylim[2], err=-1);
  
  if ( is.null( zlim ) )
    lrfLegend( xlim, ylim, range( lam$z ) )
  else
    lrfLegend( xlim, ylim, zlim )

  if( is.null( main ) ){
    if( SampNum != -1 )
      main <- paste("Concentration for Posterior Sample", SampNum);
    else
      main <- "Posterior Expected Concentration";
  }  
  title( main );
  
  if(geo){
    if( !is.null( xlim ) && !is.null( ylim ) )
      doMaps(xlim,ylim, area, county=county);
    else
      doMaps(xl,yl, area, county=county);
  }
  
  if(ShowDat)  ShowDat(MyDat);

  if(nchar(ps))
    dev.off();

  invisible(lam);
}

rescale <- function( x, currentMark, newMark ){
  # rescale the x vector so that the two elements of the vector
  # currentMark are mapped to the two elements of newMark

  y <- ( x - currentMark[1] ) / diff( range( currentMark ) );  # Scale the x coordinates down
  y <- y * diff( range( newMark ) ) + newMark[1];  # Scale the x coords back up

  return( y );
}

lrfLegend <- function( xlim, ylim, zlim ){
  # Add the legend
  xmin = xlim[1]
  xmax = xlim[2]
  ymin = ylim[1]
  ymax = ylim[2]
  leg.loc.x = c( xmax - 0.22*diff(xlim), xmax - 0.05*diff(xlim) );
  #leg.loc.x = c( xmin, xmin + 0.3*(xmax-xmin) );
  leg.loc.y = c( ymin + 0.1*(ymax-ymin) , ymin + 0.17*(ymax-ymin) );
  image.legend(z = zlim, x=leg.loc.x, y=leg.loc.y, cex.axis = 1);
}

stdevImage <- function(
  McmcFile = "lrf",  # The base name for the files containing the
                   # posterior samples.
  geo = FALSE,     # Whether to treat the data as geographical when plotting.
  nxy=100,         # The number of grid points for the prediction maps.
                   # The square root is the size of the grid on one side.
  ps="",           # The name of the postscript file to generate.  If ps="",
                   # then a graphical window is opened and no file is generated.
  county=F,     # The names of any states for which county boundaries should
                   # be shown.
  ShowDat = F,     # Whether to add the data to the map
  MyDat = NULL,    # If ShowDat = T, the data to be plotted on the contour plot.
  zlim = NULL,     # For an image plot this can be used to specify the endpoints of
                   # the color range.
  col = NULL,     # The colors to be used in an image plot.  default is image.palette()
  xlim = NULL,     # The xlim and ylim arguments can be used to override the scaling / 
  ylim = NULL,     # centering of the bounding box used by the MCMC for the graphical
                   # display.  For instance, the MCMC may have used units of tenths of 
                   # degrees, and the graphical display should perhaps be in units of
                   # degrees.  In such a case, set xlim and ylim to be the bounding box
                   # used by the MCMC in the desired units.  The translation is then
                   # done automatically.
  xlab = NULL,     # The x axis label.  If not specified, defaults are created based on
                   # the value of "geo" and the limits of the bounding box for the study area.
  ylab = NULL,     # The y axis label.  If not specified, defaults are created.
  main = NULL,     # The image title.  If not specified, a default is created.
  ...              # Additional graphical arguments for the call to image()
){
  
  #######################################################################
  #
  # Create an image plot of the posterior mean surface.
  # McmcFile is the base (root) name for the MCMC output files.
  # The MCMC files should be named <McmcFile>Out.FL and <McmcFile>Out.VL.  There
  # should also be a log file <McmcFile>Out.LG containing the log
  # information from the MCMC run.
  #

  win  = regMatch(casefold(getId()$os),"windows");  # Running under windows?
  DatLG = paste(McmcFile,"Out.LG",sep="");     # Log file
  if(!file.exists(DatLG))
    stop(paste("Unable to read lrf log file",DatLG));
  MyId  = dget(DatLG);                       # Misc stuff from log
  bdy  = MyId$etc$bdy;
  xl = bdy@xlim;
  yl = bdy@ylim;
  lam = setGrid(bdy, nreq=nxy);
  nx = length(lam$x);
  ny = length(lam$y);
  xx = rep(lam$x,rep(ny,nx));
  yy = rep(lam$y,nx);
  
  area = (xl[2] - xl[1]) * (yl[2] - yl[1]);
  lam$z <- stdevSurface(McmcFile = McmcFile, x = xx, y = yy);
  lam$z = t(matrix(lam$z, nrow=ny, ncol=nx));
  # if lam$z is outside of the interval given by zlim, truncate.
  if( !missing( zlim ) ){
    lam$z = apply( lam$z, c(1,2), max, zlim[1] );
    lam$z = apply( lam$z, c(1,2), min, zlim[2] );
  }

  if(nchar(ps)) {
    postscript(paste(ps,"-i.eps",sep=""),hori=TRUE, color.p=TRUE);
    setAspect(bdy,0.95); }
  else if(!win)  { setAspect(bdy); }
  par(mar=c(5,5,4,2));

  if( !is.null( xlim ) && !is.null( ylim ) ){
    lam$x <- rescale( lam$x, xl, xlim );
    lam$y <- rescale( lam$y, yl, ylim );

    if( !is.null( MyDat ) ){
      MyDat$x <- rescale( MyDat$x, xl, xlim );
      MyDat$y <- rescale( MyDat$y, yl, ylim );
    }
  }

  if( is.null( xlab ) || is.null( ylab ) ){
    if(geo) {
      xlab="Longitude";         ylab="Latitude";
    } else if(mean(abs(xl)) >1000) {
      xlab="Easting (1000's)";  ylab="Northing (1000's)";
    } else {
      xlab="X";         ylab="Y";
    }
  }

  # Create the arguments for the image function call.
  # xaxs="i" and yaxs="i" specify that no extra whitespace
  # should be left around the image plot
  image.args = list( x=lam$x, y=lam$y, z = lam$z,
    xlab=xlab, ylab=ylab, cex=2,
    xaxs = "i", yaxs = "i",  
    ... );  

  if( !missing( zlim ) )
    image.args$zlim = zlim;
  if( !missing( col ) )
    image.args$col = col;
  image.args$xlim <- xlim;
  image.args$ylim <- ylim;
  do.call( "image", image.args);

  # the image plotting writes over the axis lines, so
  # redraw these lines
  abline(v=xlim[1], err=-1);
  abline(v=xlim[2], err=-1);
  abline(h=ylim[1], err=-1);
  abline(h=ylim[2], err=-1);

  # Add the legend
  if ( is.null( zlim ) )
    lrfLegend( xlim, ylim, range( lam$z ) )
  else
    lrfLegend( xlim, ylim, zlim )

  if( is.null( main ) )
    main <- "Posterior Standard Deviation of the Concentration"
  title(main);

  if(geo){
    if( !is.null( xlim ) && !is.null( ylim ) )
      doMaps(xlim,ylim, area, county=county);
    else
      doMaps(xl,yl, area, county=county);
  }

  if(ShowDat)  ShowDat(MyDat);

  if(nchar(ps))
    dev.off();

  invisible(lam);
}

percentAboveImage <- function(
  McmcFile = "lrf",  # The base name for the files containing the
                   # posterior samples.
  geo = FALSE,     # Whether to treat the data as geographical when plotting.
  nxy=100,         # The number of grid points for the prediction maps.
                   # The square root is the size of the grid on one side.
  ps="",           # The name of the postscript file to generate.  If ps="",
                   # then a graphical window is opened and no file is generated.
  county=F,        # The names of any states for which county boundaries should
                   # be shown.
  ShowDat = F,     # Whether to add the data to the map
  MyDat = NULL,    # If ShowDat = T, the data to be plotted on the contour plot.
  zlim = NULL,     # For an image plot this can be used to specify the endpoints of
                   # the color range.
  col = NULL,      # The colors to be used in an image plot.  default is image.palette()
  cutoff,          # The cutoff to use
  xlim = NULL,     # The xlim and ylim arguments can be used to override the scaling / 
  ylim = NULL,     # centering of the bounding box used by the MCMC for the graphical
                   # display.  For instance, the MCMC may have used units of tenths of 
                   # degrees, and the graphical display should perhaps be in units of
                   # degrees.  In such a case, set xlim and ylim to be the bounding box
                   # used by the MCMC in the desired units.  The translation is then
                   # done automatically.
  xlab = NULL,     # The x axis label.  If not specified, defaults are created based on
                   # the value of "geo" and the limits of the bounding box for the study area.
  ylab = NULL,     # The y axis label.  If not specified, defaults are created.
  ...              # Additional graphical arguments for the call to image()
){
  
  #######################################################################
  #
  # Create an image plot of the posterior probability of exceeding a certain
  # threshold over the region
  # McmcFile is the base (root) name for the MCMC output files.
  # The MCMC files should be named <McmcFile>Out.FL and <McmcFile>Out.VL.  There
  # should also be a log file <McmcFile>Out.LG containing the log
  # information from the MCMC run.
  #

  win  = regMatch(casefold(getId()$os),"windows");  # Running under windows?
  DatLG = paste(McmcFile,"Out.LG",sep="");     # Log file
  if(!file.exists(DatLG))
    stop(paste("Unable to read lrf log file",DatLG));
  MyId  = dget(DatLG);                       # Misc stuff from log
  bdy  = MyId$etc$bdy;
  xl = bdy@xlim;
  yl = bdy@ylim;
  lam = setGrid(bdy, nreq=nxy);
  nx = length(lam$x);
  ny = length(lam$y);
  xx = rep(lam$x,rep(ny,nx));
  yy = rep(lam$y,nx);
  
  area = (xl[2] - xl[1]) * (yl[2] - yl[1]);
  lam$z <- percentAbove(McmcFile = McmcFile, x = xx, y = yy, cutoff = cutoff);
  lam$z = t(matrix(lam$z, nrow=ny, ncol=nx));

  # if lam$z is outside of the interval given by zlim, truncate.
  if( !missing( zlim ) ){
    lam$z = apply( lam$z, c(1,2), max, zlim[1] );
    lam$z = apply( lam$z, c(1,2), min, zlim[2] );
  }

  if(nchar(ps)) {
    postscript(paste(ps,"-i.eps",sep=""),hori=TRUE, color.p=TRUE);
    setAspect(bdy,0.95); }
  else if(!win)  { setAspect(bdy); }
  par(mar=c(5,5,4,2));

  if( !is.null( xlim ) && !is.null( ylim ) ){
    lam$x <- rescale( lam$x, xl, xlim );
    lam$y <- rescale( lam$y, yl, ylim );

    if( !is.null( MyDat ) ){
      MyDat$x <- rescale( MyDat$x, xl, xlim );
      MyDat$y <- rescale( MyDat$y, yl, ylim );
    }
  }

  if( is.null( xlab ) || is.null( ylab ) ){
    if(geo) {
      xlab="Longitude";         ylab="Latitude";
    } else if(mean(abs(xl)) >1000) {
      xlab="Easting (1000's)";  ylab="Northing (1000's)";
    } else {
      xlab="X";         ylab="Y";
    }
  }

  image.args = list( lam,xlab=xlab,ylab=ylab,cex=2, ... );  # The arguments for the image function call
  if( !missing( zlim ) )
    image.args$zlim = zlim;
  if( !missing( col ) )
    image.args$col = col;
  do.call( "image", image.args);
  
  # Add the legend
  if ( is.null( zlim ) )
    lrfLegend( range( lam$x ), range( lam$y ), range( lam$z ) )
  else
    lrfLegend( range( lam$x ), range( lam$y ), zlim )

  title( paste( "Posterior Probability of Exceeding", cutoff, "mg/L" ) );
  
  if(geo){
    if( !is.null( xlim ) && !is.null( ylim ) )
      doMaps(xlim,ylim, area, county=county);
    else
      doMaps(xl,yl, area, county=county);
  }

  if(ShowDat)  ShowDat(MyDat);

  if(nchar(ps))
    dev.off();

  invisible(lam);
}


lrfPlot = function(
  McmcFile = "lrf",  # The base name for the files containing the
                   # posterior samples.
  geo = FALSE,     # Whether to treat the data as geographical when plotting.
  ps="",           # The name of the postscript file to generate.  If ps="",
                   # then a graphical window is opened and no file is generated.
  SampNum=-1,      # Which MCMC sample number to plot.  If SampNum=-1
                   # then the posterior mean is plotted, or if maps = "d"
                   # then the last sample is plotted.  Not applicable
                   # if maps = "s".  
  ShowDat = TRUE,  # Whether to add the data to the map
  MyDat = NULL     # If ShowDat = T, the data to be plotted on the image or contour plot.
){
  # Load maps library
  require(maps);
  # Read output files from MCMCLoop
  DatLG = paste(McmcFile,"Out.LG",sep="");     # Log file
  DatFL = paste(McmcFile,"Out.FL",sep="");     # Fixed-length data
  DatVL = paste(McmcFile,"Out.VL",sep="");     # Variable-length data
  if(!file.exists(DatLG))
    stop(paste("Unable to read lrf log file",DatLG));
  if(!file.exists(DatFL))
    stop(paste("Unable to read fixed-length file",DatFL));
  if(!file.exists(DatVL))
    stop(paste("Unable to read vble-length file",DatVL));
  MyId  = dget(DatLG);                       # Misc stuff from log
  bdy  = MyId$etc$bdy;
  win  = regMatch(casefold(getId()$os),"windows");  # Running under windows?
  fdat <- readBin(DatFL, what="double", n=1e6, size=8);   # Fixed-length data
  fdat <- data.frame(matrix(fdat, ncol = 4, byrow = T));
  nSamps <- dim(fdat)[1];
  if( SampNum == -1 )
    SampNum = nSamps;
  M <- fdat[SampNum,2];

  xlocs <- rep(0, M);
  ylocs <- rep(0, M);
  heights <- rep(0, M);
  o <- .C("getLrfSamp",
          DatFL = DatFL,                  DatVL = DatVL,
          sampNum = as.integer(SampNum-1),                 # C indexes from 0
          xlocs = as.double(xlocs), ylocs = as.double(ylocs),
          heights = as.double(heights), nAtoms = as.integer(M) );  

  lrf = Lrf( dm = Meas( xy = Vec(o$xlocs, o$ylocs), z = o$heights ), bdy = bdy);
  
  if(nchar(ps)) {
    postscript(paste(ps,"-d.eps",sep=""),
               hori=TRUE, color.p=TRUE);
    setAspect(bdy,0.95);
  } else {
    if(!win) setAspect(bdy);
  }
  if(length(lrf)>0) show(lrf);             # Mass pts as circles;
  title(paste("Mixture Locations from MCMC Iteration #", SampNum));
  if(nchar(ps)) dev.off();
  invisible(lrf);
}


 
ShowDat = function(dat, frac=c(.25,.75), cuts = NULL,
  xVar = "x", yVar = "y", zVar = "z", cex.points = 0.5,
  cex.legend = 0.8, col.points = c("purple", "red", "chartreuse"),
  units = "") {
  # Show wells; highlight top and bottom fractions, or cut according to the
  # specified values
  x <- dat[,xVar]
  y <- dat[,yVar]
  z <- dat[,zVar]
  if(length(z)>0) {
    if (missing( cuts ))
      cuts = quantile(z, frac);
    lo  = (z < cuts[1]); hi  = (z > cuts[2]);
    mid = !(lo|hi);
    if(any(mid))points(x[mid], y[mid],
                       pch=4, col= col.points[2], cex = cex.points);        # Circled + (7 for x)
    if(any(lo)) points(x[lo],  y[lo],
                       pch= 1, col= col.points[3], cex = cex.points);     # Open circle
    if(any(hi)) points(x[hi],  y[hi],
                       pch=6, col= col.points[1], cex=cex.points);     # Filled circle
  }
  # Create a legend
  key( text = list( c( paste(">", cuts[2], units), "mid-range",
           paste("<", cuts[1], units) ), cex = cex.legend ),
       points = list( pch = c(6, 4, 1), col = col.points,
         cex = cex.points), 
       );
}

Pred = function(lrf, x,y, dis) {
  # Matrix of predicted values at points (y,x) on a grid
  nx = length(x);
  ny = length(y);
  xx = rep(x,rep(ny,nx));
  yy = rep(y,nx);
  matrix( PredNoGrid( lrf, xx, yy, dis ), nrow=ny,ncol=nx);
}

PredNoGrid = function(lrf, x,y, dis) {
  # Vector of predicted values at points (y,x) not on a grid
  if(lrf@bdy@Geo) {                          # Geodetic (lat/lon) coords?
    R   = 6378.137;                          # Earth's radius in km.
    myf = function (x, y) { cos((x-y)*pi/180); };
  # See lrf-Main for higher-precision version of Geo distance formula...
    costh = cos(y*pi/180) %o% cos(lrf@y*pi/180) *
      outer(x, lrf@x, myf) + sin(y*pi/180) %o% sin(lrf@y*pi/180);
    d2 = (R*acos(costh))^2;                  # Squared dist in km^2
  } else {                                   # Easy for Cartesian coords:
    d2 = (outer(x, lrf@x, "-"))^2 + (outer(y, lrf@y, "-"))^2;
  }
  c(exp(-0.5*d2/dis^2) %*% lrf@z);
}

traceToFile <- function(
  samps,            # A vector of samples for the parameter to be plotted.
  psName = NULL,    # The base name of the postscript file to write to
  ylab = "parameter value",
  main = ""
){

#####################################################################
#
# Write a traceplot to a file.
#
  win  = regMatch(casefold(getId()$os),"windows");  # Running under windows?  
  if( !is.null(psName) ) postscript(paste(psName,"-t.eps",sep=""),
                           hori=TRUE, color.p=TRUE);
  par(mar=c(5,5,4,2));
  nIter = length(as.vector(samps));
  samps <- mcmc(samps);
  traceplot(samps, ylab = ylab, main = main, cex = 2, xlab = "iteration");
  lines((1:nIter),cumsum(samps)/seq(along=samps),lty=3, col=3);
  if( !is.null(psName) ) dev.off();
  return();
}

mHist <- function(
  McmcFile,          # The base name of the MCMC out files
  psName = NULL      # The base name of the postscript file to write to

){

#####################################################################
#
# Create a histogram of samples of M, and superimpose the prior for M.
#

  EM <- expectedM(McmcFile = McmcFile);
  print( paste( "EM = ", EM ) );
  mSamps <- mSamps(McmcFile = McmcFile);

  histToFile(samps = mSamps, psName = psName, xlab = "M",
             main = "Histogram for M (Jump Counts)", density = "dpois",
             densArgs = EM, int = T, vertLineLoc = EM);
  return();
}

mSamps <- function(McmcFile){
  # Read output file from MCMCLoop
  DatFL = paste(McmcFile,"Out.FL",sep="");     # Fixed-length data
  if(!file.exists(DatFL))
    stop(paste("Unable to read fixed-length file",DatFL));
  fdat = readBin(DatFL, what="double", n=1e6, size=8);           # Fixed-length data
  fdat <- data.frame(matrix(fdat, ncol = 4, byrow = T));
  Ms   = fdat[,2];                           # Lengths ("M")
  return(Ms);
}

zsumSamps <- function(McmcFile){
  # Read output file from MCMCLoop
  DatFL = paste(McmcFile,"Out.FL",sep="");     # Fixed-length data
  DatVL = paste(McmcFile,"Out.VL",sep="");     # Variable-length data
  if(!file.exists(DatFL))
    stop(paste("Unable to read fixed-length file",DatFL));
  if(!file.exists(DatVL))
    stop(paste("Unable to read variable-length file",DatVL));
  fdat = read.table(DatFL,skip=1);           # Fixed-length data
  Ms   = fdat[,2];                           # Lengths ("M")
  vdat = read.table(DatVL,head=TRUE);        # Variable-length data
  grps = as.factor(rep(1:length(Ms),Ms));    # Group ID as factor
  gdat = split(vdat,grps);                   # Ragged List
  zsum = unlist(lapply(gdat,function(x){sum(x[,3])}));
                             # zsum is the sum of the jump heights for each
                             # iteration of the MCMC.
  return(Ms);
}

expectedM <- function(McmcFile){

  # Read output file from MCMCLoop
  DatLG = paste(McmcFile,"Out.LG",sep="");     # Log file
  if(!file.exists(DatLG))
    stop(paste("Unable to read log file",DatLG));
  MyId <- dget(DatLG);                          # Misc stuff from log    
  agam  = MyId$Pri$agam;
  rgam  = MyId$Pri$rgam;
  bdy  = MyId$etc$bdy
  xl = bdy@xlim;
  yl = bdy@ylim;
  area = (xl[2] - xl[1]) * (yl[2] - yl[1]);
  frac = MyId$etc$frac;
  eps = -log(1-frac)/rgam;      # About frac/Par$rgam, if frac small
  EM   = agam*area*e1(rgam*eps);
  return(EM);
}

expectedZsum <- function(McmcFile){

  # Read output file from MCMCLoop
  DatLG = paste(McmcFile,"Out.LG",sep="");     # Log file
  if(!file.exists(DatLG))
    stop(paste("Unable to read log file",DatLG));
  MyId <- dget(DatLG);                          # Misc stuff from log    
  agam  = MyId$Pri$agam;
  rgam  = MyId$Pri$rgam;
  bdy  = MyId$etc$bdy
  xl = bdy@xlim;
  yl = bdy@ylim;
  area = (xl[2] - xl[1]) * (yl[2] - yl[1]);
  frac = MyId$etc$frac;
  eps = -log(1-frac)/rgam;      # About frac/Par$rgam, if frac small
  Ez   = agam * area * exp(-rgam*eps)/rgam;
  return(Ez);
}

zsumHist <- function(
  McmcFile,          # The base name of the MCMC out files
  psName = NULL      # The base name of the postscript file to write to

){

#####################################################################
#
# Create a histogram of samples of the z sums, and superimpose the
# approximate prior mean for the z sums.
#

  Ezsum <- expectedZsum(McmcFile = McmcFile);
  print( paste( "Ez = ", Ezsum ) );
  zsumSamps <- zsumSamps(McmcFile = McmcFile);

  histToFile(samps = zsumSamps, psName = psName, xlab = "Z-sum",
             main = "Histogram of Jump Sums", vertLineLoc = Ezsum);
  return();
}

histToFile <- function(
  samps,            # A vector of samples for the parameter to be plotted.
  psName = NULL,    # The base name of the postscript file to write to
  xlab = "parameter value",
  main = "",
  density = NULL,
  densArgs = NULL,
  int = F,
  vertLineLoc = NULL
){

#####################################################################
#
# Write a histogram of parameter values to a file.  If !is.null(density),
# then a density function is superimposed on the histogram.  Then density
# gives the name of the S-PLUS density function, and densArgs gives a
# vector of arguments for that function.  If int = T, then the density
# is taken to be point masses at integral values, and otherwise it is
# taken to be a continuous function.
# If !is.null(vertLineLoc), then a vertical line is drawn at vertLineLoc.
#

  win  = regMatch(casefold(getId()$os),"windows");  # Running under windows?  
  if( !is.null(psName) ) postscript(paste(psName,"-h.eps",sep=""),
                           hori=TRUE, color.p=TRUE);
  par(mar=c(5,5,4,2));
  rv = hist(samps,prob=TRUE,plot=FALSE);
  xl = range(rv$breaks);
  yl = range(0, rv$counts);
  if(!is.null(density)) {
    densArgs$x = unique(samps);
    yl = range(yl,do.call(density,densArgs))
  }
  hist(samps,xlab=xlab,yaxt="n",prob=TRUE,plot=TRUE,main = main, cex=2);

  if(!missing(density)) {
    if(int) {
      xl = round(xl);
      x = seq(xl[1], xl[2]);
    } else {
      x = seq(xl[1], xl[2], , 101);
    }
    densArgs$x = x;
    y = do.call(density,densArgs);
    if(int) {
      lines(x,y,type="h",col=3);
      points(x,y,col=3);
    } else {
      lines(x,y,type="l",col=3);
    }
  }
  if(!is.null(vertLineLoc)) {
    if(vertLineLoc < xl[1] || vertLineLoc > xl[2])
      print(paste("Note: vertLineLoc =", dbl2g(vertLineLoc,10,4),
                  "is outside posterior histogram range"));
    else
      abline(v=vertLineLoc,lty=2,col=3);
  }
  if(!is.null(psName)) dev.off();
  return();
}


doMaps = function(xl, yl, area, county=F) {
  # Load maps library
  require(maps);
  
  map(database="state",col=1,add=TRUE,
      xlim = range(xl), ylim=range(yl));
  if(county)
    map(database="county",col=3,add=TRUE,lty=2,
      xlim = range(xl), ylim=range(yl));
}



getLocsInRegions <- function(database, states, nxy, xlim, ylim){
  # get the regions defined by the given database and within the given states.
  # Obtain nxy locations by sampling uniformly within each region,
  # and create a vector of the x and y coordinates of all of these locations.
  # Check that each region overlaps with the bounding box given by bdy;
  # if not, do not record that region.
  # Store the polygon defining each of the valid regions for mapping
  # purposes.

  require(maps);
  # get the ID numbers of the areas over which to average
  regionNums <- mapname( database, states );
  nRegions <- length(regionNums);

  regions = list();
  regNames = list();
  x = matrix( 0, nrow = 0, ncol = nxy );
  y = matrix( 0, nrow = 0, ncol = nxy );

  xLocs <- rep(0, nxy);
  yLocs <- rep(0, nxy);  

  for( i in (1:nRegions) ){
    # Obtain the coordinates of the polygon corresponding to the ith region.
    # The following code is copied from the map function in the maps library.
    # First, turn the polygon number into a list of polyline numbers
    line = mapgetg( database, regionNums[i], fill=T );
    # turn the polyline numbers into x and y coordinates
    coords = mapgetl( database, line$number );
    # remove NA breaks in coords
    coords[c("x","y")] = makepoly(coords, line$size, T);
    # create a polygon with the coordinates of this region
    thisRegion = Poly( Vec( x=coords$x, y=coords$y ) );

    # get the coordinates of a box around the region
    xmin = min(coords$x);
    xmax = max(coords$x);
    ymin = min(coords$y);
    ymax = max(coords$y);

    # limit the box to the boundaries of the posterior
    # area
    if (xmin < xlim[1])
      xmin = xlim[1];
    if (xmax > xlim[2])
      xmax = xlim[2];
    if (ymin < ylim[1])
      ymin = ylim[1];
    if (ymax > ylim[2])
      ymax = ylim[2];

    # check that the box overlaps the posterior area
    if ((xmin < xlim[2]) && (xmax > xlim[1])
        && (ymin < ylim[2]) && (ymax > ylim[1])){

      # if it does, record the polygon for the region
      regions <- c( regions, list( coords[c("x", "y")] ) );
      regNames <- c( regNames, list( names( regionNums )[i] ) )

      # generate nxy locations uniformly within the region
      # via rejection sampling
      nSamps = 0;

      while (nSamps < nxy) {
        # generate a location uniformly within the box
        xLoc = xmin + runif(1) * (xmax - xmin);
        yLoc = ymin + runif(1) * (ymax - ymin);
        
        # check whether it is inside the region
        if ( inPoly( thisRegion, Vec(xLoc, yLoc) ) ){
          # If so, add it to the list of locations in the
          # region.  
          nSamps = nSamps + 1;
          xLocs[nSamps] = xLoc;
          yLocs[nSamps] = yLoc;
        }
      }
      x <- rbind( x, xLocs );
      y <- rbind( y, yLocs );
    }
  }

  return( list( regions = regions, x = x, y = y, regNames = regNames ) );
}

regionAvg <- function(
  McmcFile = "lrf",  # The base name for the files containing the
                   # posterior samples.
  skip=0,          # The number of iterations to skip (for burn-in) at
                   # the beginning of the input file.
  database = "county",   # which database to use in the maps library
  states = c("mary","penn","virg", "west", "new york", "new jersey",
    "delaware", "north carolina"), 
                   # for which states to evaluate the averages
  nxy = 10,        # how many locations to sample for each region
  ps = "",
  ShowDat = F,  # Whether to add the data to the map
  MyDat = NULL,    # If ShowDat = T, the data to be plotted on the image or contour plot.
  xlim = NULL,     # The xlim and ylim arguments can be used to override the scaling / 
  ylim = NULL,     # centering of the bounding box used by the MCMC for the graphical
                   # display.  For instance, the MCMC may have used units of tenths of 
                   # degrees, and the graphical display should perhaps be in units of
                   # degrees.  In such a case, set xlim and ylim to be the bounding box
                   # used by the MCMC in the desired units.  The translation is then
                   # done automatically.
  cutoffs = NULL,  # The two cutoffs used to distinguish the three colors corresponding
                   # to low, medium, and high regional averages.  Default is c(0.25, 0.75).
  xlab = NULL,     # The x axis label.  If not specified, defaults are created based on
                   # the value of "geo" and the limits of the bounding box for the study area.
  ylab = NULL,     # The y axis label.  If not specified, defaults are created.
  ...              # Any graphical parameters for a call to par() after opening
                   # the graphics device and before plotting.
  ){

  # Read the posterior samples files
  DatLG = paste(McmcFile,"Out.LG",sep="");     # Log file
  DatFL = paste(McmcFile,"Out.FL",sep="");     # Fixed-length data
  DatVL = paste(McmcFile,"Out.VL",sep="");     # Variable-length data
  if(!file.exists(DatLG))
    stop(paste("Unable to read lrf log file",DatLG));
  if(!file.exists(DatFL))
    stop(paste("Unable to read fixed-length file",DatFL));
  if(!file.exists(DatVL))
    stop(paste("Unable to read vble-length file",DatVL));

  # Get the boundary of the posterior inference area in the posterior
  # samples and the scale of the kernel.
  MyId  = dget(DatLG);
  bdy  = MyId$etc$bdy;

  # sample nxy locations uniformly in each region
  if( !is.null( xlim ) && !is.null( ylim ) )
    locs = getLocsInRegions( database = database, states = states,
      nxy = nxy, xlim = xlim, ylim = ylim );
  else
    locs = getLocsInRegions( database = database, states = states,
      nxy = nxy, xlim = bdy@xlim, ylim = bdy@ylim );

  regions <- locs$regions;
  x <- locs$x;
  y <- locs$y;
  x <- as.vector( x );
  y <- as.vector( y );
  if( !is.null(xlim) && !is.null(ylim) ){
    x <- rescale( x, xlim, bdy@xlim );
    y <- rescale( y, ylim, bdy@ylim );
    MyDat$x <- rescale( MyDat$x, bdy@xlim, xlim );
    MyDat$y <- rescale( MyDat$y, bdy@ylim, ylim );
  }

  # get the posterior mean level for each location
  nLocs <- length(x);
  z <- meanSurface(McmcFile, x, y, -1);

  # calculate the average over each region
  nRegions <- length(regions);
  z <- matrix( z, ncol = nxy );
  regAvg <- rowMeans( z );

  # plot the average levels.
  if( nchar(ps) )
    postscript( paste(ps, ".eps", sep=""), hori=TRUE, color.p=TRUE );

  # Create the map
  if( is.null( xlab ) || is.null( ylab ) ){
    xlab="Longitude";         ylab="Latitude";
  }
  par(...);
  if( !is.null( xlim ) && !is.null( ylim ) )
    plot(xlim, ylim, xlab = xlab, ylab = ylab, type = "n",
         xaxs = "i", yaxs = "i");    # ensures that no extra space is left around
                         # the xlim and ylim
  else
    plot(bdy@xlim, bdy@ylim, xlab = xlab, ylab = ylab, type = "n",
         xaxs = "i", yaxs = "i");    # ensures that no extra space is left around
                         # the xlim and ylim

  # Fill in each region
  if( is.null( cutoffs ) )
    cutoffs = quantile(regAvg, probs = c(0.25, 0.75), na.rm=T)
  for (i in (1:nRegions)){

    if (!is.na(regAvg[i])){
      if( regAvg[i] < cutoffs[1] )  # lowest 
        color = "yellow";
      else if ( regAvg[i] > cutoffs[2] )  # highest
        color = "red";
      else
        color = "orange";
  
      polygon(regions[[i]], col=color);
    }
  }
  # Add the regional boundaries
  for (i in (1:nRegions)){
    polygon(regions[[i]], col="black", density = 0, lty = 3);
  }
  # Add in darker state boundaries
  map( database = "state", states=states, col = "black", add = T, lwd = 2);
  abline(v=xlim[1], lwd=3);
  abline(v=xlim[2], lwd=3);
  abline(h=ylim[1], lwd=3);
  abline(h=ylim[2], lwd=3);
  
  # Create a legend
  key( text=c( paste(">", cutoffs[2], "mg/L"), "mid-range",
           paste("<", cutoffs[1], "mg/L") ),
       rectangles = list(size=1, col = c("red", "orange", "yellow")),
       border = T, x = xlim[1] + 0.02*diff(xlim) , y = ylim[2] - 0.02*diff(xlim) );

  if(ShowDat)  ShowDat(MyDat);

  if( nchar(ps) )
    dev.off();
  invisible(regAvg);
}

regionPercAbove <- function(
  McmcFile = "lrf",  # The base name for the files containing the
                   # posterior samples.
  skip=0,          # The number of iterations to skip (for burn-in) at
                   # the beginning of the input file.
  database = "county",   # which database to use in the maps library
  states = c("mary","penn","virg", "west", "new york", "new jersey",
    "delaware", "north carolina"), 
                   # for which states to evaluate the averages
  nxy = 10,        # how many locations to sample for each region
  ps = "",
  xlim = NULL,     # The xlim and ylim arguments can be used to override the scaling / 
  ylim = NULL,     # centering of the bounding box used by the MCMC for the graphical
                   # display.  For instance, the MCMC may have used units of tenths of 
                   # degrees, and the graphical display should perhaps be in units of
                   # degrees.  In such a case, set xlim and ylim to be the bounding box
                   # used by the MCMC in the desired units.  The translation is then
                   # done automatically.
  cutoff,          # The cutoff.
  probCutoffs,     # The probability levels to delimit use of red, yellow, and green.
                   # A vector of length 2.
  xlab = NULL,     # The x axis label.  If not specified, defaults are created based on
                   # the value of "geo" and the limits of the bounding box for the study area.
  ylab = NULL,     # The y axis label.  If not specified, defaults are created.
  ...              # Any graphical parameters for a call to par() after opening
                   # the graphics device and before plotting.
  ){
  #############################################################################
  ##
  ## A plot of the percentage of posterior samples above a certain level, averaged by region.
  ##
  
  # Read the posterior samples files
  DatLG = paste(McmcFile,"Out.LG",sep="");     # Log file
  DatFL = paste(McmcFile,"Out.FL",sep="");     # Fixed-length data
  DatVL = paste(McmcFile,"Out.VL",sep="");     # Variable-length data
  if(!file.exists(DatLG))
    stop(paste("Unable to read lrf log file",DatLG));
  if(!file.exists(DatFL))
    stop(paste("Unable to read fixed-length file",DatFL));
  if(!file.exists(DatVL))
    stop(paste("Unable to read vble-length file",DatVL));

  # Get the boundary of the posterior inference area in the posterior
  # samples and the scale of the kernel.
  MyId  = dget(DatLG);
  bdy  = MyId$etc$bdy;

  # sample nxy locations uniformly in each region
  if( !is.null( xlim ) && !is.null( ylim ) )
    locs = getLocsInRegions( database = database, states = states,
      nxy = nxy, xlim = xlim, ylim = ylim );
  else
    locs = getLocsInRegions( database = database, states = states,
      nxy = nxy, xlim = bdy@xlim, ylim = bdy@ylim );

  regions <- locs$regions;
  x <- locs$x;
  y <- locs$y;
  x <- as.vector( x );
  y <- as.vector( y );
  if( !is.null(xlim) && !is.null(ylim) ){
    x <- rescale( x, xlim, bdy@xlim );
    y <- rescale( y, ylim, bdy@ylim );
  }
  
  # get the posterior mean level for each location
  nLocs <- length(x);
  z <- percentAbove( McmcFile, x, y, cutoff );
  
  # calculate the average over each region
  nRegions <- length(regions);
  z <- matrix( z, ncol = nxy );
  regAvg <- rowMeans( z );

  # plot the average levels.
  if( nchar(ps) )
    postscript( paste(ps, ".eps", sep=""), hori=TRUE, color.p=TRUE );

  # Create the map
  if( is.null( xlab ) || is.null( ylab ) ){
    xlab="Longitude";         ylab="Latitude";
  }
  par(...);
  if( !is.null( xlim ) && !is.null( ylim ) )
    plot(xlim, ylim, xlab = xlab, ylab = ylab, type = "n",
         xaxs = "i", yaxs = "i");    # ensures that no extra space is left around
                         # the xlim and ylim
  else
    plot(bdy@xlim, bdy@ylim, xlab = xlab, ylab = ylab, type = "n",
         xaxs = "i", yaxs = "i");    # ensures that no extra space is left around
                         # the xlim and ylim

  # Fill in each region
  if( is.null( probCutoffs ) )
    probCutoffs = quantile(regAvg, probs = c(0.25, 0.75), na.rm=T)
  for (i in (1:nRegions)){

    if (!is.na(regAvg[i])){
      if( regAvg[i] < probCutoffs[1] )  # lowest 
        color = "yellow";
      else if ( regAvg[i] > probCutoffs[2] )  # highest
        color = "red";
      else
        color = "orange";
  
      polygon(regions[[i]], col=color);
    }
    #print( (locs$regNames)[[i]] );
    #print(regAvg[i]);
  }
  # Add the regional boundaries
  for (i in (1:nRegions)){
    polygon(regions[[i]], col="black", density = 0, lty = 3);
  }
  # Add in darker state boundaries
  map( database = "state", states=states, col = "black", add = T, lwd = 2);
  abline(v=xlim[1], lwd=3);
  abline(v=xlim[2], lwd=3);
  abline(h=ylim[1], lwd=3);
  abline(h=ylim[2], lwd=3);

  # Create a legend
  key( text=c( paste(">", probCutoffs[2]*100, "%"), "mid-range",
           paste("<", probCutoffs[1]*100, "%") ),
       rectangles = list(size=1, col = c("red", "orange", "yellow")),
       border = T, x = xlim[1] + 0.02*diff(xlim) , y = ylim[2] - 0.02*diff(xlim) );

  if( nchar(ps) )
    dev.off();
  invisible(regAvg);
}

getMcmc <- function(
  McmcFile = "lrf",  # The base name for the files containing the
                     # posterior samples.
  coda.loc = NULL,   # the directory for the coda library
  getZsums = F       # Whether to sum the atom heights and return
                     # the resulting sums as a variable in the mcmc
                     # samples
  ){

  #######################################################################
  # getMcmc
  #
  # Creates an mcmc object (used by the coda package) containing
  # the posterior samples for M (the number of atoms in the levy process)
  # and the outcome precision.
  #

  # Read output files from MCMCLoop
  DatLG = paste(McmcFile,"Out.LG",sep="");     # Log file
  DatFL = paste(McmcFile,"Out.FL",sep="");     # Fixed-length data
  DatVL = paste(McmcFile,"Out.VL",sep="");     # Variable-length data
  if(!file.exists(DatLG))
    stop(paste("Unable to read lrf log file",DatLG));
  if(!file.exists(DatFL))
    stop(paste("Unable to read fixed-length file",DatFL));
  if(!file.exists(DatVL))
    stop(paste("Unable to read variable-length file",DatVL));

  MyId <- dget(DatLG);                          # Misc stuff from log    
  fdat <- readBin(DatFL, what="double", n=1e6, size=8);   # Fixed-length data
  fdat <- data.frame(matrix(fdat, ncol = 4, byrow = T));
  nDat <- MyId$etc$N;                           # The # of data points
  nSamps <- dim(fdat)[1];                       # The # of posterior samples
  if (nDat > 0)
    rmse <- sqrt(fdat[,3]/nDat);
  else
    rmse <- rep(0, nSamps);

  if (getZsums){
#    vdat = read.table(DatVL,head=TRUE);        # Variable-length data
#    Ms   = fdat[,2];                           # Lengths ("M")
#    grps = as.factor(rep(1:length(Ms),Ms));    # Group ID as factor
#    gdat = split(vdat,grps);                   # Ragged List
#    zsum = unlist(lapply(gdat,function(x){sum(x[,3])}));
    zsum <- rep( 0, nSamps );
    o <- .C("zsums",
      DatFL = DatFL,                  DatVL = DatVL,
      zsum = as.double(zsum) );  

    zsum = o$zsum;
                             # zsum is the sum of the jump heights for each
                             # iteration of the MCMC.
  }
  if( !is.element( "coda", search() ) ){
    if( is.null( coda.loc ) )
      library(coda);
    else
      library(coda, lib.loc = coda.loc);
  }

  mcmcDat <- fdat[,c(2,4)];
  mcmcDat$rmse <- rmse;
  vars <- c("M", "pre", "RMSE")
  if (getZsums){
    mcmcDat$zsum <- zsum
    vars <- c(vars, "zsum")
  }

  lrfmcmc <- mcmc(as.matrix(mcmcDat));  # as.matrix is only necessary here
                             # because of a bug in the mcmc function when run
                             # in Linux
  
  varnames(lrfmcmc) <- vars;

    
  lrfmcmc
}
