int2str <- function(x, w) {
  ##############################################################
  #
  # For internal use.  
  #
  # Return length w string representation of integer x

  if (x > 0) {
    len.x = ceiling(log(1e-9 +x)/log(10));
  } else if (x < 0) {
    len.x = ceiling(log(1e-9 -x)/log(10))+1;
  } else {
    len.x = 1;
  }
  if(w>len.x) {
    pad = paste(rep(' ',w-len.x),collapse='');
  } else pad = '';
  paste(pad, x, sep='');
}


int2strs <- function(x, w) {
  ##############################################################
  #
  # For internal use.  
  #
  # Return length w string representation of integer vector x
  out = ''
  for (i in 1:length(x)) {
    out = paste(out,int2str(x[1],w),sep='');
    x   = x[-1];
  }
  return(out);
}


dbl2str <- function(x, w, d) {
  ##############################################################
  #
  # For internal use.  
  #
  # Return length w string representation of double x
  i = trunc(x);
  f = floor(10^d * abs(x-i));
  if (f > 0) {
    len.f = ceiling(log(1e-9+f)/log(10));
  } else if (f < 0) {
    len.f = ceiling(log(1e-9-f)/log(10))+1;
  } else len.f = 1;
  if(d>0) {
    z = paste(rep(0,d-len.f),collapse="");
    paste(int2str(i,w-d-1),'.',z,f,sep='');
  } else
    int2str(i,w-d);
}


dbl2strs <- function(x, w, d) {
  ##############################################################
  #
  # For internal use.  
  #
  # Return length w string representation of dbl vector x
  out = ''
  for (i in 1:length(x)) {
    out = paste(out,dbl2str(x[1],w,d),sep='');
    x   = x[-1];
  }
  return(out);
}


dbl2g <- function(x, w, d) {
  ##############################################################
  #
  # For internal use.  
  #
  # Return length w string representation of double x
  ax   = abs(x);
  if((0.01 < ax && ax < 100.0) || x==0) return(dbl2str(x,w,d));
  pow = floor(log(ax)/log(10));
  x = x / 10^pow;
  paste(dbl2str(x,w-5,d-1),"E",pow,sep="");
}


str2str <- function(x, w) {
  ##############################################################
  #
  # For internal use.  
  #
  # Return length w string representation of string x
  x = substring(x,1,w);
  pad = paste(rep(' ', w-nchar(x)), collapse='');
  paste(pad,x,sep='');
}


hms2h = function(x) {
  ##############################################################
  #
  # For internal use.  
  #
  # Converts hhmmss or ddmmss data to floating-pt hours (or degrees)
  h = floor(x/1e4);
  m = floor((x%%1e4)/1e2);
  s = x%%1e2;
  h+(m+s/60)/60;
}


sec2dhms = function(s) {
  ##############################################################
  #
  # For internal use.  
  #
  # Converts floating-pt sec to dd hh mm ss
  x = s;
  d = s %/% 86400;  # Days
  s = s %%  86400;
  h = s %/%  3600;  # Hours
  s = s %%   3600;
  m = s %/%    60;  # Minutes
  s = s %%     60;
  s = round(s);     # Seconds
  paste(if(x > 86400) paste(d,"days "),
        if(x >  3600) paste(int2str(h,2),"hours "),
        if(x >    60) paste(int2str(m,2),"minutes "),
        paste(int2str(s,2)," seconds"),sep="");  
}


yymmddhhmm = function(yymmdd, hhmm=0) {
  ##############################################################
  #
  # For internal use.  
  #
  # Converts yymmdd and hhmmss date/time to floating-pt Julian date
  # The built-in function "timeDate(jul=...)" recovers yy,mo,dd, etc.
  # and probably would replace this func, for suitable "format".
## Something's wrong here when yymmdd can have NA's.  
  yy = floor(yymmdd/1e4);
  ok = (yy<10) & !is.na(yy);
  if(any(ok)) yy[ok] <- yy[ok] + 2000;  # 2-digit format, 21st c?
  ok = (yy<100) & !is.na(yy);
  if(any(ok)) yy[ok] <- yy[ok] + 1900;  # 2-digit format, 20th c?
  mo = floor((yymmdd%%1e4)/1e2);
  dd = yymmdd%%1e2;
  hh = floor(hhmm/1e2);
  mi = hhmm%%1e2;
  julian(mo,dd,yy) + (hh+mi/60)/24;
}


getAspect = function(bdy) {
  ##############################################################
  #
  # For internal use.  
  #
  # Report aspect ratio, width divided by height
  if(bdy@Geo) {
    xbar = mean(bdy@xlim);     ybar = mean(bdy@ylim); 
    dx = GeoDis(bdy@xlim[1],ybar, bdy@xlim[2],ybar);
    dy = GeoDis(xbar,bdy@ylim[1], xbar,bdy@ylim[2]);
  } else {
    dx = diff(bdy@xlim);  dy = diff(bdy@ylim);
  }
  if(dx<=0 || dy<=0) stop("getAspect: Non-positive x,y dims");
  return(dx/dy);
}


setAspect = function(bdy, shrink) {
  ##############################################################
  #
  # For internal use.  
  #
  # Make plot aspect ratio match data aspect ratio:
  if(bdy@Geo) {
    xbar = mean(bdy@xlim);     ybar = mean(bdy@ylim); 
    dx = GeoDis(bdy@xlim[1],ybar, bdy@xlim[2],ybar);
    dy = GeoDis(xbar,bdy@ylim[1], xbar,bdy@ylim[2]);
  } else {
    dx = diff(bdy@xlim);  dy = diff(bdy@ylim);
  }
  if(dx<=0 || dy<=0) stop("setAspect: Non-positive x,y dims");
  pin  = par("pin");                  # Current plot size, inches
  wide = pin[1]*dy < pin[2]*dx;
  if(wide) pin[2] = pin[1] * dy/dx;
  else     pin[1] = pin[2] * dx/dy;
  if(!missing(shrink)) pin = pin*shrink;
  par(pin=pin);
  invisible(pin);
}


setGrid = function(poly, nreq=1, nx=1, ny=1) {
  ##############################################################
  #
  # For internal use.  
  #
  # Return arrays of x and y values and grid of z values for image()
  xl=poly@xlim;  dx = diff(xl);
  yl=poly@ylim;  dy = diff(yl);
  if(dx<=0 || dy<=0) stop("setGrid: Non-positive x,y lengths");
  if(!is.numeric(nreq) || nreq < 0) nreq=1;
  if(missing(nx)) nx = round(sqrt(nreq * dx / dy));
  if(missing(ny)) ny = round(sqrt(nreq * dy / dx));
  x = seq(xl[1]+dx/(2*nx),xl[2]-dx/(2*nx),,nx);
  y = seq(yl[1]+dy/(2*ny),yl[2]-dy/(2*ny),,ny);
  z = matrix(0,nrow=ny,ncol=nx);   # note z[i,j] ~ y[i], x[j]
  n = ny*nx;                       # *Approximately* nreq
  invisible(list(x=x, y=y, z=z, nx=nx, ny=ny));
}

getId = function() {
  ##############################################################
  #
  # For internal use.  
  #
  # Gather info about user, system, date, environment, etc. to save with
  # program output, making it easier to keep track of ouptut files.
  # Try to isolate R- and S-, Windows- and Unix- dependencies.

  when = date();
  is.S = extends(class(version),"Sversion");   # "simple.list" in R
  if(is.S) {
    lang = "S-Plus";
    what = platform();
    switch(what,
           "WIN386" =
           {
             where = dos("hostname");
             who   = dos("whoami");    # Not sure how to do this w/o cygwin
             os    = dos("ver")[2];
           },
           "LINUX" =, "SUNOS5_SPARC" = # Would work with other unix's...
           {
             where = unix("hostname");
             who   = unix("whoami");
             os    = unix("uname -rps");
           },
           stop("getId: Uh oh, I only know WIN386, LINUX & SunOS"));
    ver = version;
  } else {  # for R
    lang = "R";
    info  = Sys.info();
    names(info)=NULL;
    what  = info[1];   # sysname
    where = info[4];   # nodename
    who   = info[7];
    ver   = version;
    ver   = paste(ver[11]," [Version ",ver[6],".",ver[7],", ",ver[8],
      "-",ver[9],"-",ver[10], "]",sep="")
    switch(what,
           "Windows" =
           {
             os    = system("cmd /c ver",int=TRUE,inv=TRUE)[2];
           },
           "FreeBSD" =, "SunOS" = # Add other unix's if needed
           {
             os    = system("uname -rps",int=TRUE);
           },
           stop("getId: Uh oh, I only know about Win, BSD, and SunOS"));
    
  }
  return(list("lang"=lang, "ver"=ver, "os"=os, "mach"=where,
              "arch"=what, "who"=who, "when"=when));
}


PredETA = function(Itn,ItnMax,dSav,times,Elapsed ) {
  ##############################################################
  #
  # For internal use.  
  #
  # Construct string with predicted end time

  j = 2+(Itn %/% dSav);       # Number of time-step data pts so far
  times[j,"t"] = Elapsed;     # Present Time
  times[j,"s"] = ItnMax-Itn;  # Remaining steps
  coef = summary(lm("t~s", times[max(1,j-10):j,]))$coeff[1,c(1,2)];
  dt   = coef[1]-Elapsed;     # Predicted add'l secs needed
  se   = coef[2];             # Standard error of prediction
  fin  = timeDate(date(),in.format="%w %m %d %H:%M:%S %Z %Y",
         format="%Y-%02m-%02d %02I:%02M:%02S %p");
  msec = fin@.Data[[2]] + round(1000*dt);
  days = fin@.Data[[1]] + msec %/% 86400000;
  msec = msec %% 86400000;
  fin@.Data[[1]] = as.integer(days);
  fin@.Data[[2]] = as.integer(msec);
  fin@time.zone="EST/EDT";    # Override default ("GMT" for us)
  return(list(fin = fin, dt = dt, times = times));
}


dhms2s = function(str) {
  ##############################################################
  #
  # For internal use.  
  #
  # Interpret string like "3 days 4 hours" as number of seconds

  secs=0;
  if(regMatch(str,"d")) {
    loc  = regMatchPos(str,"d[^0-9]*");
    secs = secs + 86400 * as.integer(substring(str,1,loc[1]-1));
    str  = substring(str,loc[2]+1);
  }
  if(regMatch(str,"h")) {
    loc  = regMatchPos(str,"h[^0-9]*");
    secs = secs + 3600 * as.integer(substring(str,1,loc[1]-1));
    str  = substring(str,loc[2]+1);
  }
  if(regMatch(str,"m")) {
    loc  = regMatchPos(str,"m[^0-9]*");
    secs = secs + 60 * as.integer(substring(str,1,loc[1]-1));
    str  = substring(str,loc[2]+1);
  }
  if(regMatch(str,"[0-9]")) {
    loc  = regMatchPos(str,"[0-9]*");
    secs = secs + as.integer(substring(str,loc[1],loc[2]));
  }
  return(secs);
}
