\name{lrfRun}
\alias{lrfRun}
\title{Fit the Spatial Moving-Average Model}
\description{Fit the spatial moving-average model
  described in Woodard, Wolpert, and O'Connell (2009,JABES).}
\usage{
lrfRun( Dat = NULL, OutFile = "lrf", Bdy, ItnMax = 1e6,
  ItnBrn = 1e5, JmpMax = 500, TimMax = 600, SavMax = min(1e4, ItnMax),
  pX = c(pre=.10,bet=.00,add=.25,gam=.50,rem=.15),
  dX = c(dnew=1.0,dmvmag=0.5,dmvloc=1.0),
  PriPar  = list(apre = 3, rpre = 1, dist = 0.5, agam = 3, rgam = 1),
  frac = 0.010, printStats = T )
}
\arguments{
  \item{Dat }{The data, as a data frame containing the numeric columns:
   x & y: location coordinates (e.g. in degrees), and
   z: concentration values.  If unspecified, will sample from the prior.}
 \item{OutFile }{Output file base name.  The posterior samples will be
saved in the binary files
   OutFile"Out.FL" and OutFile"Out.VL".  A text log file will also be
   created, called OutFile"Out.LG".}
 \item{Bdy }{Set from which mass pts are drawn; the results of a
 call to the function "BBox".  The default is to take the range of x and y and
 expand by 10 percent in each direction.}
\item{ItnMax }{The maximum number of iterations of the Markov chain,
  after burn-in.}
\item{ItnBrn }{The number of burn-in iterations of the Markov chain.}
\item{JmpMax }{The maximum number of mixture components to be sampled in
  the initial configuration.  This will not affect the limiting
  distribution of the Markov chain; the default usually works fine for
  this argument.}
\item{TimMax }{Not yet implemented.  The maximum run time.}
\item{SavMax }{The approximate number of times to save the state of the
  Markov chain to the output files.  The thinning of the chain
  is taken to be \code{floor( ItnMax / SavMax )}. }
\item{pX }{A vector specifying the probability of each move type.
  Should have length 5 and sum to 1.  The second element corresponds to
  the probability of updating the beta parameter; since this parameter
  has not yet been implemented this element of pX should be zero. The
  remaining elements of the vector correspond to: 1. updating the precision,
  3. adding a mixture component, 4. changing the location and scale of
  a mixture component, and 5. removing a mixture component.}
\item{dX }{A vector specifying the proposal step size for each type
  of proposal.  The elements correspond to: 1. generating the magnitude of a new
  mixture component, 2. updating the magnitude of a mixture component,
  and 3. updating the location of a mixture component.}
\item{PriPar }{A list of the values for the prior constants. The
  elements are equal to the following, which are named as in Woodard,
  Wolpert, and O'Connell (2009): 1. alpha_sigma, 2. rho_sigma, 3. d,
  4. alpha, and 5. rho.}
\item{frac }{Specifies the truncation level epsilon, via the fraction
  of the mass that is omitted.  This is typically chosen to be small,
  and can be manipulated to help control the number of mixture
  components in the posterior samples.}
\item{printStats }{Whether to print summary statistics about the
  MCMC run, such as the run time and the number of attempted and
  successful moves of each type.}
}
\examples{

######################## Explore the prior distribution.
# get the range of the data, in kilometers (eastings/northings)
xlim = range(gwNit$x);
ylim = range(gwNit$y);
dist = 40;

# add a buffer around the edges to mitigate the edge effects of the model
xlim[1] = xlim[1] - 3*dist;
xlim[2] = xlim[2] + 3*dist;
ylim[1] = ylim[1] - 3*dist;
ylim[2] = ylim[2] + 3*dist;
# add the same buffer to the degree-coordinates range.  We need
# the (buffered) range in both coordinate systems in order to do the
# mapping correctly
percAddX <- (3*dist) / diff( range( gwNit$x ) );
percAddY <- (3*dist) / diff( range( gwNit$y ) );
xlimOrig <- range( gwNit$Lon )
ylimOrig <- range( gwNit$Lat )
xlimOrig[1] <- xlimOrig[1] - diff( xlimOrig ) * percAddX;
xlimOrig[2] <- xlimOrig[2] + diff( xlimOrig ) * percAddX;
ylimOrig[1] <- ylimOrig[1] - diff( ylimOrig ) * percAddY;
ylimOrig[2] <- ylimOrig[2] + diff( ylimOrig ) * percAddY;

# Draw from the prior.  This takes less than a minute
# on one typical system, but the run time can be reduced
# by lowering ItnMax to e.g. 1e5.
lrfRun(ItnMax = 1e7, Bdy = BBox(xlim, ylim),
       ItnBrn = 1e6, frac = 0.01,    
       SavMax = 1e4, PriPar  = list(apre = 0.39, rpre = 0.0098, 
                   dist = dist,
                   agam = 1.07e-4,   
                   rgam = 0.244),
       OutFile="prior1",
       dX = c(0.3, 2, 200));

# convergence diagnostics.  Requires the coda package.
# If coda is installed in a non-standard directory,
# specify the lib.loc argument when loading coda.
\dontrun{
library(coda)
lrfmcmc <- getMcmc("prior1", getZsums = T)
par( mfrow = c(2,2) )
java.graph()
traceplot(lrfmcmc)
}
# create a plot of one sample.  Change SampNum to any number
# between 1 and the value of SavMax in the call to lrfRun
# in order to see plots of other prior samples.
# Use the java.graph device, since the default device
# generates unnecessary warnings.
java.graph()
a <- meanImage("prior1",ShowDat=F, MyDat = gwNit, nxy = 1000, geo = T, zlim = c(0,12),
               xlab = "Longitude", ylab = "Latitude", main = "",
               xlim = xlimOrig, ylim = ylimOrig, SampNum=7800)

# Create a plot of the same sample, showing the locations and magnitudes
# of the mixture components.
java.graph()
lrfPlot("prior1", SampNum = 7800)



######################## Explore the posterior distribution.
# Fit the model.  Below we run JUST A SHORT TRIAL RUN.
# For interpretable results, run with ItnMax = 1e6,
# ItnBrn = 1e5, SavMax = 1e4.
lrfRun( Dat = gwNit, Bdy = BBox(xlim, ylim),
       ItnMax = 1e2, ItnBrn = 1e2, 
    frac = 0.01,    
    SavMax = 1e2, 
    PriPar  = list(apre = 0.39, rpre = 0.0098, 
                   dist = dist,
                   agam = 1.07e-4,   
                   rgam = 0.244),
       dX = c(0.3, 2, 200),       
       OutFile="test");

# convergence diagnostics.  Requires the coda package.
# If coda is installed in a non-standard directory,
# specify the lib.loc argument when loading coda.
\dontrun{
library(coda)
lrfmcmc <- getMcmc("test", getZsums = T)
java.graph()
par( mfrow = c(2,2) )
traceplot(lrfmcmc)
autocorr.plot(lrfmcmc)
}

# create plots of the mean, stan. dev., and prob. of exceedance.  These
# are only meaningful if you have done a full run of the Markov chain
# as described above.
java.graph()
a <- meanImage("test",ShowDat=T, MyDat = gwNit, nxy = 1000, geo = T, zlim = c(0,12),
               xlab = "Longitude", ylab = "Latitude", main = "",
               xlim = xlimOrig, ylim = ylimOrig)
java.graph()
a <- stdevImage("test",ShowDat=T, MyDat = gwNit, nxy = 1000, geo = T, zlim = c(0,5),
               xlab = "Longitude", ylab = "Latitude", main = "",
               xlim = xlimOrig, ylim = ylimOrig)
java.graph()
a <- percentAboveImage("test",ShowDat=F, MyDat = gwNit, nxy = 1000, geo= T,
               cutoff = 10, zlim = c(0,0.15),
               xlab = "Longitude", ylab = "Latitude",
               xlim = xlimOrig, ylim = ylimOrig)

# create plots of one sample
java.graph()
a <- meanImage("test",ShowDat=F, MyDat = gwNit, nxy = 1000, geo = T, zlim = c(0,12),
               xlab = "Longitude", ylab = "Latitude", main = "",
               xlim = xlimOrig, ylim = ylimOrig, SampNum=10)
java.graph()
lrfPlot("test", SampNum = 10)

# look at average concentrations over the counties, and the probability
# of exceeding a threshold, averaged by county.  The plots are
# WRITTEN TO A POSTSCRIPT FILE called test.ps.  In order to generate
# the plot in S-PLUS, remove the ps="test" argument.
# These calls are for demonstration purposes; to generate the graphs
# used in the paper increase the values of nxy, which are the number
# of sampled locations in each county.
regionAvg("test", nxy = 2, ps="test", xlim = xlimOrig, ylim = ylimOrig, ShowDat = F, MyDat = gwNit,
          cutoffs = c(1, 5), fin = c(6.2,7.5))

# This call can take > 1 min.
regionPercAbove("test", nxy = 10, ps = "test", xlim = xlimOrig, ylim = ylimOrig,
          cutoff = 10, probCutoffs = c(0.02, 0.08), fin = c(6.2,7.5))


}
