
#include <vector>
#include <fstream>
#include "LevyInfer.h"

#define SQR(x) ((x)*(x))
using TNT::Array1D;

LevyInfer::LevyInfer(
  char *DatFL, char *DatVL,
  double dist){

  DatFL_ = DatFL;
  DatVL_ = DatVL;
  dist_ = dist;
}

// Take a matrix that is represented as a vector of nRows Array1D objects, where 
// the length of each Array1D is the number of columns, and average the rows.
Array1D<double> rowAvg( 
  const std::vector< Array1D<double> > &mat
  ){

  Array1D<double> rowSum( mat[0].dim(), 0.0 );

  int i, nRows = mat.size();
  for(i=0; i<nRows; i++){
    rowSum = rowSum + mat[i];
  }
  return( rowSum / (double) nRows );
}

int getSamp( FILE *FLfile, int *itn, int *nAtoms, double *sse, double *pre ){

  double tempDbl[4];

  int nRead = fread( tempDbl, sizeof(double), 4, FLfile );
  
  (*itn) = (int) tempDbl[0];
  (*nAtoms) = (int) tempDbl[1];
  (*sse) = tempDbl[2];
  (*pre) = tempDbl[3];

  return(nRead);
}

int getAtom( FILE *VLfile, double *x, double *y, double *height){

  double tempDbl [3];

  int nRead = fread( tempDbl, sizeof(double), 3, VLfile );
  (*x) = tempDbl[0];
  (*y) = tempDbl[1];
  (*height) = tempDbl[2];

  return(nRead);
}

  /**************************************************************************
   **  Get the height of a kernel centered at (x,y), with height=
   **  height, at the specified locations.
   **/  
Array1D<double> LevyInfer::kernelHeight(
  const Array1D<double> &xlocs, const Array1D<double> &ylocs,
  double x, double y, double height ){

  Array1D<double> sqrDist, sqrDistTemp, kernelHeight;

  #ifdef DEBUG1
    std::cout << "x=  " << x << " y= " << y << std::endl;
  #endif
  // calculate the squared distance of each location from the atom
  sqrDist = xlocs - x;
  #ifdef DEBUG1
    std::cout << "sqrDist: " << sqrDist;
  #endif
  sqrDist = sqrDist^2;
  #ifdef DEBUG1
  std::cout << "sqrDist: " << sqrDist;
  #endif
  sqrDistTemp = ylocs - y;
  sqrDistTemp = sqrDistTemp^2;
  sqrDist = sqrDist + sqrDistTemp;
  #ifdef DEBUG1
    std::cout << "sqrDist: " << sqrDist;
  #endif
  // calculate the height of the kernel at each location
  kernelHeight = -0.5*sqrDist / SQR(dist_);
  #ifdef DEBUG1
    std::cout << "kernelHeight: " << kernelHeight;
  #endif
  kernelHeight = TNT::exp( kernelHeight );
  #ifdef DEBUG1
    std::cout << "kernelHeight: " << kernelHeight;
  #endif
  kernelHeight = kernelHeight * height;
  #ifdef DEBUG1
    std::cout << "kernelHeight: " << kernelHeight;
  #endif

  return( kernelHeight );
}

Array1D<double> LevyInfer::meanVals( 
  const Array1D<double> &xlocs, const Array1D<double> &ylocs,
  int sampNum
){

  int nLocs = xlocs.dim();
  Array1D<double> z( nLocs );

  FILE *FLfile = fopen( DatFL_, "r" );
  FILE *VLfile = fopen( DatVL_, "r" );
 
  double sse, pre, x, y, height;
  int itn, nAtoms, i;

  if( sampNum >= 0 )
    return sampSurf( xlocs, ylocs, sampNum );

  // Create a vector that is the sum of the surface height over the samples,
  // for each location.
  Array1D<double> surfHeightSum( nLocs, 0.0 );
  Array1D<double> sampSurfHeight( nLocs ), kernHeight;
  
  // Read in each sample saved in the fixed-length file 
  //while( fscanf( FLfile, "%d %d %f %f", &itn, &nAtoms, &sse, &pre ) == 4 ){
  int nSamps = 0;

  #ifdef DEBUG1
    std::cout << "dist = " << dist_ << std::endl;
  #endif
  
  while( getSamp( FLfile, &itn, &nAtoms, &sse, &pre ) == 4 ){

    #ifdef DEBUG1
      std::cout << itn << " " << nAtoms << " " << sse << " " << pre << std::endl;
    #endif

    // Reset sampSurfHeight to zero
    sampSurfHeight = 0;
    for(i=0; i<nAtoms; i++){
      if( getAtom( VLfile, &x, &y, &height ) < 3 ){
	MESSAGE "Data in variable length file does not match fixed length file." ERROR;
      }
      kernHeight = kernelHeight( xlocs, ylocs, x, y, height );
      sampSurfHeight = sampSurfHeight + kernHeight;
    }
    surfHeightSum = surfHeightSum + sampSurfHeight;
    nSamps++;
  }
  z = surfHeightSum / (double) nSamps;

  fclose(FLfile);
  fclose(VLfile);

  return(z);
}



Array1D<double> LevyInfer::stdevSurf( 
  const Array1D<double> &xlocs, const Array1D<double> &ylocs
){

  int nLocs = xlocs.dim();
  Array1D<double> z( nLocs );

  Array1D<double> meanSurf = meanVals( xlocs, ylocs ),
    squaredDiffSum( nLocs, 0.0 );

  FILE *FLfile = fopen( DatFL_, "r" );
  FILE *VLfile = fopen( DatVL_, "r" );
 
  double sse, pre, x, y, height;
  int itn, nAtoms, i;

  // Create a vector that is the sum of the surface height over the samples,
  // for each location.
  if( meanSurf.dim() != nLocs )
    MESSAGE "incorrect length of vector meanSurf" ERROR;

  Array1D<double> sampSurfHeight( nLocs ),
    kernHeight, sampDiff;
  
  // Read in each sample saved in the fixed-length file 
  //while( fscanf( FLfile, "%d %d %f %f", &itn, &nAtoms, &sse, &pre ) == 4 ){
  int nSamps = 0;

  #ifdef DEBUG1
    std::cout << "dist = " << dist_ << std::endl;
  #endif
  
  while( getSamp( FLfile, &itn, &nAtoms, &sse, &pre ) == 4 ){

    #ifdef DEBUG1
      std::cout << itn << " " << nAtoms << " " << sse << " " << pre << std::endl;
    #endif

    // Reset sampSurfHeight to zero
    sampSurfHeight = 0;
    for(i=0; i<nAtoms; i++){
      if( getAtom( VLfile, &x, &y, &height ) < 3 ){
	MESSAGE "Data in variable length file does not match fixed length file." ERROR;
      }
      kernHeight = kernelHeight( xlocs, ylocs, x, y, height );
      sampSurfHeight = sampSurfHeight + kernHeight;
    }
    sampDiff = meanSurf - sampSurfHeight;
    sampDiff = sampDiff * sampDiff;
    squaredDiffSum = squaredDiffSum + sampDiff;
    nSamps++;
  }
  z = squaredDiffSum / (double) nSamps;
  z = z^0.5;

  fclose(FLfile);
  fclose(VLfile);

  return(z);
}


Array1D<double> LevyInfer::percentAbove( 
  const Array1D<double> &xlocs, const Array1D<double> &ylocs,
  double cutoff
){

  int nLocs = xlocs.dim();

  FILE *FLfile = fopen( DatFL_, "r" );
  FILE *VLfile = fopen( DatVL_, "r" );
 
  double sse, pre, x, y, height;
  int itn, nAtoms, i, j;

  Array1D<double> sampSurfHeight( nLocs ), kernHeight,
    nSampsAbove( nLocs, 0.0 );
  
  #ifdef DEBUG1
    std::cout << "dist = " << dist_ << std::endl;
  #endif
  
  // Read in each sample saved in the fixed-length file 
  int nSamps = 0;

  while( getSamp( FLfile, &itn, &nAtoms, &sse, &pre ) == 4 ){

    #ifdef DEBUG1
      std::cout << itn << " " << nAtoms << " " << sse << " " << pre << std::endl;
    #endif

    // Reset sampSurfHeight to zero
    sampSurfHeight = 0;
    for(i=0; i<nAtoms; i++){
      if( getAtom( VLfile, &x, &y, &height ) < 3 ){
	MESSAGE "Data in variable length file does not match fixed length file." ERROR;
      }
      kernHeight = kernelHeight( xlocs, ylocs, x, y, height );
      sampSurfHeight = sampSurfHeight + kernHeight;
    }
    for(j=0; j<nLocs; j++){
      if( sampSurfHeight[j] > cutoff )
	nSampsAbove[j] = nSampsAbove[j] + 1;
    }
    nSamps++;
  }

  fclose(FLfile);
  fclose(VLfile);

  return( nSampsAbove / (double) nSamps );
}

Array1D<double> LevyInfer::sampSurf( 
  const Array1D<double> &xlocs, const Array1D<double> &ylocs,
  int sampNum
){

  int nLocs = xlocs.dim();
  Array1D<double> z( nLocs );

  FILE *FLfile = fopen( DatFL_, "r" );
  FILE *VLfile = fopen( DatVL_, "r" );
 
  double sse, pre, x, y, height;
  int itn, nAtoms, i;

  Array1D<double> sampSurfHeight( nLocs, 0.0 ),
    kernHeight;
  
  // Read in each sample saved in the fixed-length file 
  int nSamps = 0;

  #ifdef DEBUG1
    std::cout << "dist = " << dist_ << std::endl;
  #endif
  
  while( getSamp( FLfile, &itn, &nAtoms, &sse, &pre ) == 4 ){

    #ifdef DEBUG1
      std::cout << itn << " " << nAtoms << " " << sse << " " << pre << std::endl;
    #endif

    for(i=0; i<nAtoms; i++){
      if( getAtom( VLfile, &x, &y, &height ) < 3 ){
	  MESSAGE "Data in variable length file does not match fixed length file." ERROR;
      }
      if (nSamps == sampNum){
        kernHeight = kernelHeight( xlocs, ylocs, x, y, height );
        #ifdef DEBUG1
          std::cout << "kernelHeight: " << kernHeight;
        #endif
        sampSurfHeight = sampSurfHeight + kernHeight;
      }
    }
    nSamps++;
  }

  fclose(FLfile);
  fclose(VLfile);

  return(sampSurfHeight);
}



Array1D<double> LevyInfer::zsums(){

  FILE *FLfile = fopen( DatFL_, "r" );
  FILE *VLfile = fopen( DatVL_, "r" );
 
  std::vector<double> zsum;

  double sse, pre, x, y, height, heightSum;
  int itn, nAtoms, i;

  // Read in each sample saved in the fixed-length file 
  //while( fscanf( FLfile, "%d %d %f %f", &itn, &nAtoms, &sse, &pre ) == 4 ){
  int nSamps = 0;
  while( getSamp( FLfile, &itn, &nAtoms, &sse, &pre ) == 4 ){

    // Reset jump height sum to zero
    heightSum = 0;
    for(i=0; i<nAtoms; i++){
      if( getAtom( VLfile, &x, &y, &height ) < 3 ){
	MESSAGE "Data in variable length file does not match fixed length file." ERROR;
      }
      heightSum += height;
    }
    zsum.push_back(heightSum);
    nSamps++;
  }
  
  // Convert to a doubles array
  Array1D<double> zsumArr(nSamps);
  for(i=0; i<nSamps; i++)
    zsumArr[i] = zsum[i];

  fclose(FLfile);
  fclose(VLfile);

  return(zsumArr);
}

/**************************************************************************
 **  Get the lrf for sample number sampNum
 **/  
void LevyInfer::getLrfSamp( int sampNum, Array1D<double> &xlocs, 
			    Array1D<double> &ylocs,
			    Array1D<double> &heights ){

  FILE *FLfile = fopen( DatFL_, "r" );
  FILE *VLfile = fopen( DatVL_, "r" );
 
  std::vector<double> zsum;

  double sse, pre, x, y, height;
  int itn, nAtoms, i;

  // Read in each sample saved in the fixed-length file 
  //while( fscanf( FLfile, "%d %d %f %f", &itn, &nAtoms, &sse, &pre ) == 4 ){
  int nSamps = 0;
  while( getSamp( FLfile, &itn, &nAtoms, &sse, &pre ) == 4 ){
    
    if(( nSamps == sampNum ) && (nAtoms != xlocs.dim()))
      MESSAGE "Data in variable length file does not match length of output vectors" ERROR;
    for(i=0; i<nAtoms; i++){
      if( getAtom( VLfile, &x, &y, &height ) < 3 ){
	MESSAGE "Data in variable length file does not match fixed length file." ERROR;
      }
      if( nSamps == sampNum ){
	xlocs[i] = x;
	ylocs[i] = y;
	heights[i] = height;
      }
    }
    nSamps++;
  }
  
  fclose(FLfile);
  fclose(VLfile);

}
