
#include "LevyMCMC.h"
#include "randGen.h"

/* debugging info: 
   To print high-level debugging info at runtime, add "-DDEBUG1" to the first line of the Makevars
   file.  To print more detailed debugging info, also add "-DDEBUG2" and possibly "-DDEBUG3" 
*/

// Define indexes for the move types
#define NmoveTypes 5
#define MV_PRE 0    // precision update
#define MV_BETA 1    // beta update
#define MV_NEWPT 2    // add an atom to the levy process
#define MV_MVPT 3     // move an atom in the levy process
#define MV_DELPT 4    // delete an atom in the levy process

// Define indexes for the move types 
// that require a scale value for the proposal
#define Nscales 3
#define SCA_DRAWPT 0   // add an atom to the levy process
#define SCA_MVMAG 1    // change an atom's magnitude
#define SCA_MVLOC 2    // change an atom's location

#define SQR(x) ((x)*(x))

using TNT::Array1D;
using TNT::Array2D;
using std::cout;
using std::endl;

extern "C"
{
  double S_STDCALL S_rpois( double lambda );
  double S_STDCALL S_rgamma( double s );
  double S_STDCALL S_runif( void );
  double S_STDCALL S_rexp( void );
}


void printAcceptProb( double logProb ){
  if (logProb > 1000)
    printf("log accept prob = infty \n" );
  else if (logProb < -1000)
    printf("log accept prob = - infty \n" );
  else 
    printf("log accept prob = %f\n", logProb );
}

LevyMCMC::LevyMCMC(
     double *datX,   double *datY,   // The data (x, y, z for each of datLen samples, where (x,y) is the location
     double *datZ,   int datLen,   // and z is the measurement)
     char *OutFL, char *OutVL,     // The names of the fixed-length and variable-length files to bes written
     double *xlim,     double *ylim,  // The bounding region within which to fit the surface.  xlim and ylim should 
                                      //be length-2 vectors
     int ItnMax,     int ItnBrn,  // ItnMax is the number of total iterations after burnin, while ItnBrn is 
                                      // the length of the burnin
     int JmpMax,     int TimMax,  // JmpMax is the max number of atoms in the fit Poisson process.  TimMax 
                                      // is the maximum length of 
                                      // time to sample, in seconds.
     int SavMax,     double *pX,    // SavMax is the maximum number of saved iterations.  The resulting thinning 
                                      // is approximately 
                                      // ItnMax / SavMax.  pX is a vector of the probabilities of proposing a move 
                                      // for each of the parameters:
                                      //     - prec: outcome (measurement) precision, conditional on a fixed z-surface
                                      //     - beta: the regression coefficients (not currently supported--should be zero)
                                      //     - birth: add a new atom
                                      //     - move: change the location and height of one of the existing atoms
                                      //     - remove: remove one of the atoms
     double *dX,       double apre,  // dX is the step size for the proposal for each of the above move types.  apre and rpre
     double rpre,     double dist,  // are the parameters of the Gamma prior distribution for the outcome (measurement) precision.
     double agam,     double rgam,  // dist is the standard deviation of the independent bivariate normal kernels of which the surface
                                      // is composed.  agam and rgam are the parameters of the Levy process prior for the spatial 
                                      // surface.
     double frac,                     // frac is the fraction of the mass to be truncated (this controls the value of epsilon)
     bool printStats )                // Whether to print summary statistics about the MCMC
				      // (e.g. acceptance rates)
{
  datX_ = Array1D<double>(datLen, datX);
  datY_ = Array1D<double>(datLen, datY);
  datZ_ = Array1D<double>(datLen, datZ);
  #ifdef DEBUG1
  cout << "datX_ = \n" << datX_ << "\ndatY = \n" << datY_ << "\ndatZ_ = \n" << datZ_;
  #endif
  datLen_ = datLen;
  OutFL_ = OutFL;
  OutVL_ = OutVL;
  xlim_ = xlim;
  ylim_ = ylim;
  ItnMax_ = ItnMax;
  ItnBrn_ = ItnBrn;
  JmpMax_ = JmpMax;  
  TimMax_ = TimMax;
  SavMax_ = SavMax;
  pX_ = Array1D<double> (NmoveTypes, pX);
  dX_= Array1D<double> (Nscales, dX);
  apre_ = apre;
  rpre_ = rpre;
  agam_ = agam;
  rgam_ = rgam;
  dist_ = dist;
  printStats_ = printStats;
  double frac_ = std::max(0.0001, std::min(0.75, frac));   // Limit range of frac
  if( rgam_ <= 0 ){
    printf("rgam specification invalid\n");
    return;
  }
  epsilon_ = -log(1-frac_)/rgam_;               // eps approximately = frac/rgam, if frac small  
}

LevyMCMC::~LevyMCMC(){}


/**************************************************************************
 **  Calculate the negative log likelihood 
 **  
 **/  
double LevyMCMC::nllh( LevyProc *levy,
           double pre, double *SSE ) {

  double startTime = tk.clockSecs();
  /*  (*SSE) = 0.0;
  for(i=0; i< datLen_; ++i) {
    x = datX_[i];
    y = datY_[i];
    z = datZ_[i];
    (*SSE) += levy->squaredError( x, y, z );
    }*/
  (*SSE) = levy->sumSquaredError( datX_, datY_, datZ_ );
  likeEvalTime += tk.clockSecs() - startTime;

  double nlgLik = ((*SSE) * pre - datX_.dim() * log(pre))/2;
  
  return( nlgLik );
}

/**************************************************************************
 **  Update the outcome precision parameter via a Gibbs full conditional 
 **  draw
 **/  
double LevyMCMC::updatePrecision( 
  LevyProc *levy,
  double *nlgLik, 
  double *sse ){

  double shape = apre_ + ((double) datX_.dim())/2,
    rate = rpre_ + (*sse)/2;

  if( (rate <= 0) || (shape <= 0) ){
    MESSAGE "rate and shape must be positive\n" ERROR;
  }
  // generate from the full conditional dist'n
  // of the observation precision sigma^-2
  double pre  = S_rgamma( shape );
  pre = pre / rate;  
  
  // calc. the new negative log-likelihood and new SSE
  (*nlgLik) = nllh( levy, pre, sse );

  return(pre);                      // Always accept Gibbs step
}

/**************************************************************************
 **
 **/  
double LevyMCMC::dLevy (
  double z, 
  bool ln) {
  // Levy density at x,y,z (or LOG density if ln=TRUE)
  double lpdf = log( agam_ / z ) - rgam_ * z;
  if (ln) return( lpdf );
  else return( exp(lpdf) );
}

/**************************************************************************
 **  Attempt to add a new atom to the Levy process.  If the addition is 
 **  accepted, update levy, nlgLik, and sse.
 **/  
bool LevyMCMC::proposeNewPoint( 
  LevyProc *levy, double pre, double *nlgLik,
  double *sse ){

  double x, y, z, pdf, nsse;
  levy->drawPoint( &x, &y, &z, &pdf, dX_[SCA_DRAWPT] );    // New (proposed) point
                                             // dX_[SCA_DRAWPT] is the scale of the 
                                             // proposal for the magnitude

  #ifdef DEBUG2
    printf("proposing x = %f, y=%f, z=%f, pdf = %f \n", x, y, z, pdf );
  #endif

  if( z == 0 ){
    MESSAGE "Error: z = 0\n" ERROR;
  }
  if( dX_[SCA_MVMAG] == 0 ){
    MESSAGE "Error: dX_[SCA_MVMAG] = 0\n" ERROR;
  }
  if( epsilon_ == 0 ){
    MESSAGE "Error: epsilon = 0\n" ERROR;
  }
  double Padd = pX_[MV_NEWPT],
    Prem = pX_[MV_DELPT]+pX_[MV_MVPT] * S_pnorm( log( epsilon_ / z ) / dX_[SCA_MVMAG] );

  int addInd = levy->addPoint( x, y, z );  // Add the new point to the Levy 
                            // process and keep its index for easy removal.
                            // Conceptually, we add the new point at a position
                            // that is chosen uniformly, but it turns out that
                            // computationally the order doesn't matter; for 
                            // efficiency purposes we therefore 
                            // appended the new point to the end of the list of 
                            // points.

  // calculate the negative log likelihood and sum of squared errors of the 
  // proposal
  double newLik = nllh( levy, pre, &nsse );
  // calculate the difference in the log posterior probabilities minus
  // the difference in the proposal probabilities
  double logH = (*nlgLik) - newLik + log( Prem / ( Padd * levy->length() ) ) - 
    pdf + dLevy(z);

  #ifdef DEBUG2
    printAcceptProb(logH);
  #endif

  double samp = S_rexp();
  if( logH + samp > 0 ) {   // accept the proposed addition
    (*sse) = nsse;          // update the sum of sq. errors
    (*nlgLik) = newLik;     // update the neg. log like.
    #ifdef DEBUG2
      printf("proposal accepted\n");
    #endif
    return (true);
  } else {
    // Reject the proposed addition.  Remove the proposed point from the 
    // Levy process.
    levy->deletePoint( addInd );
    return (false);
  }
}


/**************************************************************************
 **  Attempt to move an atom in the Levy process.  If the move is 
 **  accepted, update levy, nlgLik, and sse.
 **/
bool LevyMCMC::movePoint( 
  LevyProc *levy, double pre, double *nlgLik,
  double *sse ){
  
  int nPoints = levy->length();
  #ifdef DEBUG2
    printf("nPoints = %d\n", nPoints);
  #endif

  if (nPoints == 0)
    return(false);
  // Sample an atom for moving
  Array1D<double> equalProbs (nPoints, 1/(double)nPoints);
  #ifdef DEBUG3
    cout << "equalProbs:" << endl;
    cout << equalProbs << endl;
  #endif
  int j = rdiscrete(equalProbs);
  if( (j < 0) || (j >= nPoints) )
    MESSAGE "Choosing a nonexistent point to move\n" ERROR;

  #ifdef DEBUG2
    printf(" choosing atom %d for move\n", j);
  #endif

  // Get the location and height of the atom with index j
  double x, y, z;
  levy->getPoint( j, &x, &y, &z );

  double step = S_rnorm() * dX_[SCA_MVMAG],
    nz = z * exp(step);   // New z (atom magnitude)
  #ifdef DEBUG3
    cout << "step: " << step << "  old z: " << z << 
      "  new z: " << nz << endl;
  #endif

  if (nz < epsilon_) {
    return( deleteGivenPoint( levy, pre, nlgLik, sse, j ) );
  } else {
    // propose step in location
    double dx = S_rnorm() * dX_[SCA_MVLOC],
      dy = S_rnorm() * dX_[SCA_MVLOC];
    levy->changePoint( j, (x+dx), (y+dy), nz );
    
    // calculate the negative log likelihood and sum of squared errors of the 
    // proposal
    double nsse,
      newLik = nllh( levy, pre, &nsse );
    double logH = (*nlgLik) - newLik + rgam_ * (z - nz);
    #ifdef DEBUG2
      printAcceptProb(logH);
    #endif

    // Accept or reject
    double samp = S_rexp();
    if( logH + samp > 0 ) {   // accept the proposed addition
      (*sse) = nsse;          // update the sum of sq. errors
      (*nlgLik) = newLik;     // update the neg. log like.
      #ifdef DEBUG2
	printf("proposal accepted\n");
      #endif
      return (true);
    } else {
      // Reject the proposed addition.  Move the proposed point back to its
      // original location and height
      levy->changePoint( j, x, y, z );
      return (false);
    }
  }

}

/**************************************************************************
 **  Attempt to remove atom with indicator ind from the Levy process.  If 
 **  the removal is accepted, update levy, nlgLik, and sse.
 **/
bool LevyMCMC::deleteGivenPoint( 
  LevyProc *levy, double pre, double *nlgLik,
  double *sse, int ind ){

  // Delete the atom, saving the location and magnitude in case the 
  // proposed deletion is rejected.
  double x, y, z, nsse;
  levy->getPoint( ind, &x, &y, &z );
  #ifdef DEBUG2
    printf(" choosing atom (%f, %f, %f) for delete\n", x, y, z );
  #endif
  levy->deletePoint( ind );

  // calculate the negative log likelihood and sum of squared errors of the 
  // proposal
  double newLik = nllh( levy, pre, &nsse );

  // Calculate the density of the atom birth distribution, evaluated at
  // the deleted point
  double pdf = levy->dBirth( x, y, z, dX_[SCA_DRAWPT] );

  // Calculate the probability of proposing such a deletion and the 
  // probability of proposing the opposite addition.
  if( z == 0 ){
    MESSAGE "Error: z = 0\n" ERROR;
  }
  if( dX_[SCA_MVMAG] == 0 ){
    MESSAGE "Error: dX_[SCA_MVMAG] = 0\n" ERROR;
  }
  if( epsilon_ == 0 ){
    MESSAGE "Error: epsilon = 0\n" ERROR;
  }
  double Padd = pX_[MV_NEWPT],
    Prem = pX_[MV_DELPT]+pX_[MV_MVPT] * S_pnorm( log( epsilon_ / z ) / dX_[SCA_MVMAG] );
  
  // calculate the difference in the log posterior probabilities minus
  // the difference in the proposal probabilities
  double logH = (*nlgLik) - newLik + log( ( Padd * (levy->length()+1) ) / Prem ) +
    pdf - dLevy(z);
 
  #ifdef DEBUG2
    printAcceptProb( logH );
  #endif 

  double samp = S_rexp();
  if( logH + samp > 0 ) {   // accept the proposed addition
    (*sse) = nsse;          // update the sum of sq. errors
    (*nlgLik) = newLik;     // update the neg. log like.
    #ifdef DEBUG2
      printf("proposal accepted\n");
    #endif
    return (true);
  } else {
    // Reject the proposed deletion.  Add the proposed point back into the 
    // Levy process.
    levy->addPoint( x, y, z, ind );
    return (false);
  }
}

/**************************************************************************
 **  Attempt to remove an atom from the Levy process.  If the removal is 
 **  accepted, update levy, nlgLik, and sse.
 **/
bool LevyMCMC::deletePoint( 
  LevyProc *levy, double pre, double *nlgLik,
  double *sse ){
  
  int nPoints = levy->length();
  #ifdef DEBUG2
    printf("nPoints = %d\n", nPoints);
  #endif

  if (nPoints == 0)
    return(false);
  // Sample an atom for deletion
  Array1D<double> equalProbs (nPoints, 1/(double)nPoints);
  int j = rdiscrete(equalProbs);       // sample the point to remove

  return( deleteGivenPoint( levy, pre, nlgLik, sse, j ) );
}

/**************************************************************************
 **  Run the MCMC
 **/  
void LevyMCMC::doMCMC(){

  if (SavMax_ == 0) {
    printf("SavMax specified as zero, which is invalid.\n");
    return;
  }  

  //  Calculate the interval between state saves 
  int dSav = (int) floor( ItnMax_ / (double) SavMax_ );  
  if (dSav == 0)   dSav = 1;

  double startTime = tk.clockSecs();

  Ntry = Array1D<int> (NmoveTypes, 0);  // The number of tries and
  Nacc = Array1D<int> (NmoveTypes, 0);  //  acceptances of various move types
  
  //  Initialize the parameters
  if ((rpre_ <= 0) || (apre_ <= 0)){
    printf( "prior hyperparameter rpre or apre has an invalid value: rpre = %f, apre = %f\n", rpre_, apre_ );
    return;
  }
  double pre   = apre_ / rpre_;        // Initialize outcome precision to prior mean
  #ifdef DEBUG1
    printf( "pre = %f, epsilon = %f, ItnBrn = %d, ItnMax = %d, dSav = %d, datLen_ = %d\n", 
	    pre, epsilon_, ItnBrn_, ItnMax_, dSav, datLen_ );
    printf("OutFL = %s, OutVL = %s\n", OutFL_, OutVL_);
  #endif

  //Generate from the prior dist'n of the Levy process Gamma(ds)
  LevyProc *levy = new LevyProc(xlim_, ylim_, JmpMax_, agam_, rgam_, epsilon_, dist_); 
  levy->drawFromPrior();   
  #ifdef DEBUG1
    printf("levy process = \n");
    levy->print();
  #endif
  
  double nlgLik [1],
    sse [1];
  (*nlgLik) = DBL_MAX;                           // Force accep of 1st proposal
  
  // Calculate the initial sum of squared errors
  if (datLen_ == 0)
    (*sse) = 0;
  else
    (*sse) = sum(datZ_*datZ_);   
  #ifdef DEBUG1
    printf("initial sse = %f\n", (*sse));
  #endif
  
  //Array2D<int> times (SavMax_+2, 2);
                      // "times" will contain (# iter left, time) pairs
  //  times[0][0] = ItnMax_+ItnBrn_;
  //  times[0][1] = Start;

  int mv;
  double timRem = 0;
  bool acc;
  
  FILE *FLfile = fopen(OutFL_, "w" );
  FILE *VLfile = fopen(OutVL_, "w" );

  #ifdef DEBUG1
    printf("pX_ = %f %f %f %f %f\n", pX_[MV_PRE], pX_[MV_BETA], 
	   pX_[MV_NEWPT], pX_[MV_MVPT], pX_[MV_DELPT]);
    printf("dX_ = %f %f %f %f\n", dX_[SCA_DRAWPT],
	   dX_[SCA_MVMAG], dX_[SCA_MVLOC]);
  #endif
  
  int itn = (-ItnBrn_-1);
  double nAtoms, itnDbl;

  likeEvalTime = 0;   // Reset the time spent on likelihood evaluations to zero
  while( (itn < ItnMax_) && (timRem >= 0) ){
    
    //   Generate a proposal:
    //   Pick a parameter to update
    mv = rdiscrete(pX_);     // Which type?
    Ntry[mv] = Ntry[mv] + 1;  // Increment the number of tries for this move type

    switch (mv){
    case MV_PRE:
      pre = updatePrecision( levy, nlgLik, sse );
      #ifdef DEBUG2
	printf("updating precision to %f\n", pre);
      #endif
      Nacc[MV_PRE] += 1;   // All precision moves are accepted
      break;
    case MV_BETA:
      printf("lrfRun: No explanatory variables yet so set Pb=0\n");
      return;
      break;
    case MV_NEWPT:
      acc = proposeNewPoint( levy, pre, nlgLik, sse );
      if (acc) Nacc[MV_NEWPT] += 1;
      break;
    case MV_MVPT:
      acc = movePoint( levy, pre, nlgLik, sse );
      if (acc) Nacc[MV_MVPT] += 1;
      break;
    case MV_DELPT:
      acc = deletePoint( levy, pre, nlgLik, sse );
      if (acc) Nacc[MV_DELPT] += 1;
      break;
    default:
      printf("lrfRun: invalid move type\n");
    }
    #ifdef DEBUG1
      printf( "pre = %f, sse = %f, nlgLik = %f, levy process = \n", pre, (*sse), (*nlgLik) );
      levy->print();  fflush(stdout);
    #endif

    //    currentTime = tk.clockSecs();
    //    timRem  = Start + TimMax_ - currentTime
 
    if( ((itn % dSav) == 0) && (itn >= 0) ){
	// ---------------------   Write state to files  ------------------------
	//if(itn == 0) {
	  //	  burnEnd = tk.clockSecs();         // Init output files:
	
	  //fprintf( FLfile, "     Itn          M             SSE            Pre\n" );
	  //fprintf( VLfile, "         x          y               z\n" );
	  //Ntry = Array1D<int> (NmoveTypes, 0);        // Start over w/ # of tries
	  //Nacc = Array1D<int> (NmoveTypes, 0);        // Start over w/  # of accepts
      //}
	/*printf( "lrf: %d steps completed; %d steps (or %d seconds) remain.", itn,
	  (ItnMax_-itn), timRem );*/
	//fprintf( FLfile, "%8d      %5d      %10.4f     %10.4f\n", itn, 
	// levy->length(), (*sse), pre );
      nAtoms = (double) levy->length();
      itnDbl = (double) itn;
        
      fwrite( &itnDbl, sizeof(double), 1, FLfile );
      fwrite( &nAtoms, sizeof(double), 1, FLfile );
      fwrite( sse, sizeof(double), 1, FLfile );
      fwrite( &pre, sizeof(double), 1, FLfile );
      levy->appendToFile( VLfile );
      //      ETA = PredETA(itn,ItnMax,dSav,times,currentTime);
      //printf( "Will finish in %d seconds \n", ETA );
      //      times = ETA$times;                             # Need Assignment HERE

      //}
    }
    itn++;
  }



  // --------------------------   Wrap Up  -----------------------------
  //  fprintf( FLfile, "hello");
  fclose( FLfile );
  fclose( VLfile );
  if((itn>0) && (printStats_)){
    //    printf( "Completed %d burn-in and %d steps in %f seconds\n", ItnBrn_, itn,
    //       currentTime-Start );
    double compTime = tk.clockSecs() - startTime;
    printf( "Total computation time: %f\n", compTime );
    printf( "Computation time for likelihood evals: %f\n", likeEvalTime );
    printf( "Number of Tries: %d %d %d %d %d\n", Ntry[MV_PRE], Ntry[MV_BETA],
	    Ntry[MV_NEWPT], Ntry[MV_MVPT], Ntry[MV_DELPT]);
    printf( "Number of Accepts: %d %d %d %d %d\n", Nacc[MV_PRE], Nacc[MV_BETA],
	    Nacc[MV_NEWPT], Nacc[MV_MVPT], Nacc[MV_DELPT]);
  }
  
}


