#ifndef LEVY_MCMC_H
#define LEVY_MCMC_H


#include "tnt_array2d.h"
#include "tnt_array1d.h"
#include "tnt_array1d_utils.h"
#include "moreArray1dUtils.h"
#include "randGen.h"
#include "LevyProc.h"


using TNT::Array1D;


class LevyMCMC{
 private:
  Array1D<double> datX_, datY_, datZ_;
  int datLen_;
  char *OutFL_, *OutVL_;
  double *xlim_, *ylim_;
  int ItnMax_, ItnBrn_, 
    JmpMax_, TimMax_, SavMax_;
  Array1D<double> pX_, dX_;
  double apre_, rpre_, dist_, agam_, rgam_, epsilon_;
  double likeEvalTime;
  timeKeeper tk;

  Array1D<int> Ntry,  // The number of tries and
    Nacc;                //  acceptances of various move types
  bool printStats_;     // Whether to print MCMC summaries
			// such as acceptance rates.
  
  /**************************************************************************
   **  Calculate the negative log likelihood 
   **/  
  double nllh( LevyProc *levy, double pre, double *SSE );
  
  /**************************************************************************
   **  Update the outcome precision parameter via a Gibbs full conditional 
   **  draw
   **/  
  double updatePrecision( LevyProc *levy,
      double *nlgLike, double *sse );
  
  /**************************************************************************
   **/  
  double dLevy ( double z, bool ln=true);
  
  /**************************************************************************
   **  Attempt to add a new atom to the Levy process.  If the addition is 
   **  accepted, update levy, nlgLik, and sse.
   **/  
  bool proposeNewPoint( LevyProc *levy, double pre, double *nlgLik,
			double *sse );
  
  /**************************************************************************
   **  Attempt to move an atom from the Levy process.  If the move is 
   **  accepted, update levy, nlgLik, and sse.
   **/
  bool movePoint( LevyProc *levy, double pre, double *nlgLik,
			      double *sse );

  /**************************************************************************
   **  Attempt to remove an atom from the Levy process.  If the removal is 
   **  accepted, update levy, nlgLik, and sse.
   **/
  bool deletePoint( LevyProc *levy, double pre, double *nlgLik,
			      double *sse );

  /**************************************************************************
   **  Attempt to remove the atom with indicator ind from the Levy
   **  process.  If the removal is accepted, update levy, nlgLik, and
   **  sse.
   **/
  bool deleteGivenPoint( LevyProc *levy, double pre, double *nlgLik,
			 double *sse, int ind );
  
 public:
  LevyMCMC(
     double *datX,   double *datY,   // The data (x, y, z for each of datLen samples, where (x,y) is the location
     double *datZ,   int datLen,   // and z is the measurement)
     char *OutFL,    char *OutVL,     // The names of the fixed-length and variable-length files to be written
     double *xlim,   double *ylim,  // The bounding region within which to fit the surface.  xlim and ylim should 
                                      //be length-2 vectors
     int ItnMax,     int ItnBrn,  // ItnMax is the number of total iterations after burnin, while ItnBrn is 
                                      // the length of the burnin
     int JmpMax,     int TimMax,  // JmpMax is the max number of atoms in the fit Poisson process.  TimMax 
                                      // is the maximum length of 
                                      // time to sample, in seconds.
     int SavMax,     double *pX,    // SavMax is the maximum number of saved iterations.  The resulting thinning 
                                      // is approximately 
                                      // ItnMax / SavMax.  pX is a vector of the probabilities of proposing a move 
                                      // for each of the parameters:
                                      //     - prec: outcome (measurement) precision, conditional on a fixed z-surface
                                      //     - beta: the regression coefficients (not currently supported--should be zero)
                                      //     - birth: add a new atom
                                      //     - move: change the location and height of one of the existing atoms
                                      //     - remove: remove one of the atoms
     double *dX,     double apre,  // dX is the step size for the proposal for each of the above move types.  apre and rpre
     double rpre,    double dist,  // are the parameters of the Gamma prior distribution for the outcome (measurement) precision.
     double agam,    double rgam,  // dist is the standard deviation of the independent bivariate normal kernels of which the surface
                                      // is composed.  agam and rgam are the parameters of the Levy process prior for the spatial 
                                      // surface.
     double frac,                     // frac is the fraction of the
				      // mass to be truncated (this
				      // controls the value of
				      // epsilon)
     bool printStats = true);         // Whether to print summary
				      // statistics about the MCMC
				      // (e.g. acceptance rates)

    ~LevyMCMC();

  /**************************************************************************
   **  Run the MCMC
   **/  
  void doMCMC();

};

#endif
