#ifndef LEVYPROC_H
#define LEVYPROC_H

#include "tnt_array1d.h"
#include "tnt_array1d_utils.h"
#include "moreArray1dUtils.h"
#include "timeKeeper.h"
#include <vector>
#include <S.h>

using TNT::Array1D;
using std::vector;

class LevyProc{
 private:
  double *xlim_, *ylim_;    // Length-2 vectors indicating the
			  // boundaries of the region.
  double area;	           // The area of the region.
  double agam_, rgam_;     // The parameters of the Levy process.
  double dist_;		   // The standard deviation of the normal kernel.
  double epsilon_;         // The epsilon truncation level of the
			   // process.
  int JmpMax_;	     	   // The maximum number of atoms
  timeKeeper tk;

  vector<double> xLoc_,
    yLoc_, atomHeight_;  // xLoc_ and yLoc_
			   // are the length-nAtoms_ vectors
			   // of the atom locations and
			   // atomHeight_ is the
			   // length-nAtoms_ vector of atom
			   // heights.  These are vectors because
			   // vectors can easily have elements
			   // added or deleted.
  int nAtoms_;             // The number of atoms.

  /**************************************************************************
   **  Draw a location uniformly from the rectangular region.  Put the coords 
   **  in x and y.
   **
   **/
  void drawUnifLoc(double *x, double *y);

 public:
  LevyProc(  
    double *xlim = NULL,       // The boundaries of the region
    double *ylim = NULL,            
    int JmpMax = -1,             // The max number of atoms
    double agam = 1.0,         // Parameters of the prior for the num. of atoms.
    double rgam = 1.0,         // and the size of each atom
    double epsilon = 0.1,      // The epsilon truncation of the Levy process
			       // mass to exclude
    double dist = 1.0          // The standard deviation of the normal kernel
    );

  ~LevyProc();


  /**************************************************************************
   **  Calculate the squared distance of z from the height of the Levy
   **  process surface at the location (x,y), on the log scale.
   **  datGeo indicates whether the distance should take into account
   **  the curvature of the earth.
   **/
  double squaredError( double x, double y, double z, bool datGeo = false );

  /**************************************************************************
   **  Calculate the summed squared distance of z from the height of the Levy
   **  process surface at the location (x,y), on the log scale.
   **  datGeo indicates whether the distance should take into account
   **  the curvature of the earth.
   **/
  double sumSquaredError( const Array1D<double> &x, 
			  const Array1D<double> &y, 
			  const Array1D<double> &z, bool datGeo = false );

  /**************************************************************************
   **  Draw from the Gamma random field prior: generate random draw of size M 
   **  within the rectangular region.
   **/
  void drawFromPrior();

  /**************************************************************************
   **  Draw a new atom for the Levy process.  Draw the location (x,y)
   **  uniformly from the rectangular region, and draw the height z
   **  using the scale s.  Calculate the pdf (if ln = false) or the
   **  log-pdf (if ln = true).  Do not add the atom to the levy process.
   **/  
  void drawPoint( double *x, double *y, double *z, double *pdf, double s = 1, 
		  bool ln = true );

  /**************************************************************************
   ** Calculate the density of the atom birth proposal distribution
   **/  
  double dBirth( 
    double x, double y, double z,    // The location and height of the atom
    double s,         // The scale with which the weight of the atom would be 
                       // generated
    bool ln = true       // Whether to return the log pdf or the pdf
    );


  /**************************************************************************
   **  Add an atom at location (x,y) and with height z to the Levy
   **  process so that the atom has index ind in the new list of atoms.
   **  ind should be between -1 and length() inclusive; a value of -1
   **  is taken to be equal to length().  Return the index of the new atom.
   **/  
  int addPoint( double x, double y, double z, int ind = -1 );

  /**************************************************************************
   **  Get the location and magnitude of the atom with index ind.
   **/  
  void getPoint( int ind, double *x, double *y, double *z );

  /**************************************************************************
   **  Change the atom with index ind to location (x,y) and height z
   **/  
  void changePoint( int ind, double x, double y, double z );

  /**************************************************************************
   **  Delete the atom with index ind.
   **/  
  void deletePoint( int ind );

  /**************************************************************************
   **  The number of atoms.
   **/  
  int length(){ return (nAtoms_); };

  /**************************************************************************
   **  Write the Levy process atom locations and heights out to a
   **  file, appending to the end of the file.
   **/  
  void appendToFile( FILE *outfile );

  /**************************************************************************
   **  Write the Levy process atom locations and heights to stdout
   **/  
  void print();

};

#endif
